/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef ACTION_H
#define ACTION_H

#include <toolkit/stdmodel.h>
#include "util/metrum3.h"
#include "variant.h"

class Drawing;
class Object;
class ActionPoint;
class ObjectContext;
class Attribute;

class Action
{
public:
    Action(Drawing *drawing);
    virtual ~Action();
    Drawing *drawing() const;

    virtual void apply() = 0;
    virtual void unapply() = 0;

private:
    Drawing *const the_drawing;
};

class NewObjectAction : public Action
{
public:
    NewObjectAction(Object *object);
    void apply();
    void unapply();

private:
    Object *const object;
};

class MoveHandleAction : public Action
{
public:
    MoveHandleAction(ActionPoint *h, const Metrum3 &p);
    void apply();
    void unapply();

private:
    ActionPoint *const handle;
    Metrum3 old_pos;
    Metrum3 const new_pos;
};

class DeleteObjectAction : public Action
{
public:
    DeleteObjectAction(Object *ob);
    void apply();
    void unapply();

private:
    Object *const object;
    ObjectContext *context;
    int index;
};

class ChangeAttributeAction : public Action
{
public:
    ChangeAttributeAction(const Variant &value,
			  Attribute *attribute, Object *ob);
    void apply();
    void unapply();

private:
    Variant old_value;
    Variant const new_value;
    Attribute *const attribute;
    Object *const object;
};

class ActionGroup : public Item
{
public:
    ActionGroup(const List<Action *> &actions, Icon *icon,
		const String &name, ActionGroup *parent);
    ActionGroup(Drawing *drawing, const String &path);
    ~ActionGroup();

    ActionGroup *parent() const;
    Drawing *drawing() const;

    List<ActionGroup *> children() const;
    void addChild(ActionGroup *child);

    void apply();
    void unapply();
    bool isApplied() const;

private:
    List<Action *> const the_actions;
    ActionGroup *const the_parent;
    List<ActionGroup *> the_children;
    Drawing *const the_drawing;
    MutableStandardInterior interior;
    bool is_applied;
};

#endif
