/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef ATTRIBUTE_H
#define ATTRIBUTE_H

#include <toolkit/model_view.h>
#include "util/string.h"

class Hub;
class Variant;

class Attribute
{
public:
    Attribute(const String &name, Icon *icon = 0);
    String const name;
    Icon *const icon;
};

class AbstractAttributeItem : public Item
{
public:
    virtual bool showAttributes(const List<Attribute *> &attributes) = 0;
};

class AttributeItem : public AbstractAttributeItem
{
    Hub &hub;
public:
    AttributeItem(Hub &hub, Attribute &attribute);
    ~AttributeItem();
    bool showAttributes(const List<Attribute *> &attributes);

    bool isSet() const;
    bool hasDefault() const;
    void reset();

protected:
    Attribute &attribute;
    Variant value() const;
    void setValue(const Variant &value);
};

class StringAttributeItem : public AttributeItem
{
public:
    StringAttributeItem(Hub &hub, Attribute &attribute);
    String text() const;
    void setText(const String &value);
};

class SpinboxAttributeItem : public AttributeItem
{
public:
    SpinboxAttributeItem(Hub &hub, Attribute &attribute);
    virtual String repr() const = 0;
    virtual void setRepr(const String &str) = 0;
};

class IntegerAttributeItem : public SpinboxAttributeItem
{
public:
    IntegerAttributeItem(Hub &hub, Attribute &attribute);
    String repr() const;
    void setRepr(const String &str);
};

class MetrumAttributeItem : public SpinboxAttributeItem
{
public:
    MetrumAttributeItem(Hub &hub, Attribute &attribute);
    String repr() const;
    void setRepr(const String &str);
};

class DoubleAttributeItem : public SpinboxAttributeItem
{
public:
    DoubleAttributeItem(Hub &hub, Attribute &attribute);
    String repr() const;
    void setRepr(const String &str);
};

class SelectorAttributeItem : public AttributeItem
{
public:
    SelectorAttributeItem(Hub &hub, Attribute &attribute,
			  ItemInterior &choices);
    ItemInterior &choices;
    virtual Item *selectedItem() = 0;
    virtual void setSelectedItem(Item *value) = 0;
};

class EnumAttributeItem : public SelectorAttributeItem
{
public:
    EnumAttributeItem(Hub &hub, Attribute &attribute, ItemInterior &choices);
    Item *selectedItem();
    void setSelectedItem(Item *item);
};

class BoolAttributeItem : public SelectorAttributeItem
{
public:
    BoolAttributeItem(Hub &hub, Attribute &attribute);
    Item *selectedItem();
    void setSelectedItem(Item *item);
};

#endif
