/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef GENERIC_LINEAR_H
#define GENERIC_LINEAR_H

#include "majormode.h"
#include "object.h"
#include "action_pl.h"

class LinearMajorMode : public MajorMode
{
public:
    LinearMajorMode(Hub &hub);
    void paint(Painter &painter);
    List<ActionLine *> lines(ActionPoint *handle);

    void accept(MinorMode *minor_mode);
    void cancel();

private:
    PointSelectMinorMode startMinorMode;
    PointSelectMinorMode endMinorMode;

    virtual Object *buildObject(const Metrum3 &p0, const Metrum3 &p1) = 0;
};

class LinearObject : public Object
{
public:
    LinearObject(const Metrum3 &p0, const Metrum3 &p1, Drawing *drawing);
    void paint(Painter &painter, const Embedding &embedding);
    void paintShadow(const Metrum3 &pos, const ActionPoint *handle,
		     Painter &painter);

    List<ActionPoint *> handles();
    Metrum3 handlePos(const ActionPoint *handle) const;
    List<ActionLine *> handleLines(const ActionPoint *handle);
    void moveHandle(const Metrum3 &pos, const ActionPoint *handle);
    Metrum3 lineBase(const ActionLine *line) const;
    Metrum3 lineTangent(const ActionLine *line) const;
    ScreenDist proximity(const ScreenPos &point, View *view,
			 const Embedding &embedding) const;
    bool within(const ScreenRect &rect, View *view, bool partially,
		const Embedding &embedding) const;

    Metrum3 p0, p1;

private:
    ActionPoint startHandle, midpointHandle, endHandle;
    ObjectActionLine startNormal, midpointNormal, endNormal;
    ObjectActionLine tangent;
};

#endif
