/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#include <sys/stat.h>
#include <stdio.h>
#include <err.h>
#include <getopt.h>
#include <toolkit/toolkit.h>
#include "util/path.h"
#include "shared.h"
#include "sch_attribute.h"
#include "instance.h"
#include "../config.h"
#include "../gettext.h"

#define _(str) gettext(str)

int main(int argc, char **argv)
{
    if (setlocale(LC_ALL, "") == NULL)
	errx(EXIT_FAILURE, "setlocale: Request cannot be honored");

    if (bindtextdomain(PACKAGE, LOCALEDIR) == NULL)
	err(EXIT_FAILURE, "bindtextdomain");
    if (textdomain(PACKAGE) == NULL)
	err(EXIT_FAILURE, "textdomain");

    if (!XSupportsLocale())
	errx(EXIT_FAILURE, _("XSupportsLocale: Xlib functions are not "
			     "capable of operating under the current locale"));
    if (XSetLocaleModifiers("") == NULL)
	errx(EXIT_FAILURE, _("XSetLocaleModifiers: Invalid values given for "
			     "one or more modifier categories"));

    static struct option long_options[] = {
	{"help",        no_argument,       0, -2},
	{"version",     no_argument,       0, -3},
	{0, 0, 0, 0}
    };

    int c;

    while ((c = getopt_long(argc, argv, "", long_options, NULL)) != -1)
	switch (c) {
	case -2:
	    printf(_("Usage: %s [OPTION]... [FILE]...\n"), argv[0]);
	    printf(_("Create, modify, or examine a technical drawing\n"));
	    printf("\n");
	    printf(_(
"      --help            give this help\n"
"      --version         display version number\n"));
	    printf("\n");
	    printf(_("Report %s bugs to %s\n"),
		   PACKAGE_NAME, PACKAGE_BUGREPORT);
	    return EXIT_SUCCESS;
	case -3:
	    printf(PACKAGE_STRING "\n");
	    printf(_("Copyright (C) 2013 Roland Lutz\n"));
	    printf("\n");
	    printf(_(
"This program is free software; you can redistribute it and/or\n"
"modify it under the terms of the GNU General Public License\n"
"as published by the Free Software Foundation; either version 2\n"
"of the License, or (at your option) any later version.\n"));
	    printf("\n");
	    printf(_(
"This program is distributed in the hope that it will be useful,\n"
"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
"GNU General Public License for more details.\n"));
	    return EXIT_SUCCESS;
	default:
	    fprintf(stderr, _("Try `%s --help' for more information.\n"),
			    argv[0]);
	    return EXIT_FAILURE;
	}

    String rcdir = home_dir() + "/.xi";
    struct stat buf;

    if (stat(rcdir.data(), &buf) == -1)
	err(EXIT_FAILURE, "stat: %s", rcdir.data());
    if (!S_ISDIR(buf.st_mode))
	errx(EXIT_FAILURE, "'%s' is not a directory", rcdir.data());

    Toolkit::Connection *conn = new Toolkit::Connection(NULL);

    schematicAttributes = new SchematicAttributes();
    shared.initialize();
    instantiate(*conn);

    Toolkit::EventLoop::run();

    return EXIT_SUCCESS;
}
