/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef MAJORMODE_H
#define MAJORMODE_H

#include <toolkit/timer.h>
#include <toolkit/model_view.h>
#include "minormode.h"

class MajorModeTimer : public Toolkit::Timer
{
    MajorMode &major_mode;
public:
    MajorModeTimer(MajorMode &majorMode);
    void timeout();
};

class MajorMode : public Item
{
    MinorMode *minor_mode;
    List<Object *> the_selected;
    List<ActionPoint *> active_points;
    ActionPoint *currently_snapped_point;
    MajorModeTimer timer;

    friend class MajorModeTimer;
    void timeout();

protected:
    Hub &hub;
    void setMinorMode(MinorMode *mode, bool reset = false);
    void setSelected(const List<Object *> &selected);

public:
    MajorMode(Hub &hub);
    MinorMode *minorMode() const;
    List<Object *> selected() const;

    virtual void paint(Painter &painter);
    virtual List<ActionLine *> lines(ActionPoint *handle);

    List<ActionPoint *> activePoints() const;
    void notifySnapped(ActionPoint *point);

    virtual void accept(MinorMode *minor_mode) = 0;
    virtual void cancel() = 0;
};

class SelectMajorMode : public MajorMode
{
    SelectMinorMode selectMinorMode;
    GrabHandleMinorMode grabHandleMinorMode;
public:
    SelectMajorMode(Hub &hub);
    List<ActionLine *> lines(ActionPoint *handle);

    void accept(MinorMode *minor_mode);
    void cancel();

    void setSelected(const List<Object *> &selected);
};

class DeleteMajorMode : public MajorMode
{
    SelectMinorMode selectMinorMode;
public:
    DeleteMajorMode(Hub &hub);

    void accept(MinorMode *minor_mode);
    void cancel();
};

#endif
