/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef MINORMODE_H
#define MINORMODE_H

#include "util/string.h"
#include "util/screen.h"
#include "util/metrum3.h"

class Hub;
class MajorMode;
class Painter;
class ActionLine;
class ActionPoint;
class Object;

class MinorMode
{
    String status_message;

protected:
    Hub &hub;
    MajorMode &parent;
    virtual void accept();
    virtual void cancel();
    void setStatusMessage(const String &message);

public:
    MinorMode(Hub &hub, MajorMode &parent);
    String statusMessage() const;
    virtual void invoke();
    virtual void paint(Painter &painter);
    virtual void mousePressEvent(const ScreenPos &p, unsigned int button,
				 unsigned int buttons, unsigned int state);
    virtual void mouseMoveEvent(const ScreenPos &p,
				unsigned int buttons, unsigned int state);
    virtual void mouseReleaseEvent(const ScreenPos &p, unsigned int button,
				   unsigned int buttons, unsigned int state,
				   bool significantlyMoved);
    virtual bool snapToPoint();
    virtual bool snapToLine();
    virtual bool showPoint();
};

class PointSelectingMinorMode : public MinorMode
{
    List<ActionLine *> snappedLines;

protected:
    virtual void snapPoint(ActionPoint *point) = 0;
    virtual void snapLine(ActionLine *line, ActionPoint *point,
			  const ScreenPos &p) = 0;
    virtual void snapIntersection(ActionLine *line0, ActionLine *line1) = 0;
    virtual void freeSelect(const Metrum3 &m) = 0;

public:
    PointSelectingMinorMode(Hub &hub, MajorMode &parent);
    void mouseMoveEvent(const ScreenPos &p,
			unsigned int buttons, unsigned int state);
    void paint(Painter &painter);
};

class PointSelectMinorMode : public PointSelectingMinorMode
{
    Metrum3 the_point;
    void applyOffset();

protected:
    void snapPoint(ActionPoint *point);
    void snapLine(ActionLine *line, ActionPoint *point, const ScreenPos &p);
    void snapIntersection(ActionLine *line0, ActionLine *line1);
    void freeSelect(const Metrum3 &m);

public:
    PointSelectMinorMode(Hub &hub, MajorMode &parent);
    void invoke();
    void mousePressEvent(const ScreenPos &p, unsigned int button,
			 unsigned int buttons, unsigned int state);
    void mouseReleaseEvent(const ScreenPos &p, unsigned int button,
			   unsigned int buttons, unsigned int state,
			   bool significantlyMoved);
    bool showPoint();

    Metrum3 point();
    void setPoint(const Metrum3 &point);

    bool released;
};

class SelectMinorMode : public MinorMode
{
    bool rubber_box;
    ScreenPos start, stop;
    List<Object *> the_objects;
    ActionPoint *the_handle;
    bool select_partially_covered;

protected:
    void accept();

public:
    SelectMinorMode(Hub &hub, MajorMode &parent);
    void paint(Painter &painter);
    void mousePressEvent(const ScreenPos &p, unsigned int button,
			 unsigned int buttons, unsigned int state);
    void mouseMoveEvent(const ScreenPos &p,
			unsigned int buttons, unsigned int state);
    void mouseReleaseEvent(const ScreenPos &p, unsigned int button,
			   unsigned int buttons, unsigned int state,
			   bool significantlyMoved);

    List<Object *> objects();
    ActionPoint *handle();
};

class GrabHandleMinorMode : public PointSelectingMinorMode
{
    ActionPoint *the_handle;
    Metrum3 the_point;

protected:
    void accept();
    void snapPoint(ActionPoint *point);
    void snapLine(ActionLine *line, ActionPoint *point, const ScreenPos &p);
    void snapIntersection(ActionLine *line0, ActionLine *line1);
    void freeSelect(const Metrum3 &m);

public:
    GrabHandleMinorMode(Hub &hub, MajorMode &parent);
    void paint(Painter &painter);
    void mousePressEvent(const ScreenPos &p, unsigned int button,
			 unsigned int buttons, unsigned int state);
    void mouseReleaseEvent(const ScreenPos &p, unsigned int button,
			   unsigned int buttons, unsigned int state,
			   bool significantlyMoved);

    ActionPoint *handle();
    void setHandle(ActionPoint *handle);
    Metrum3 point();
};

#endif
