/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef NB_LUMP_H
#define NB_LUMP_H

#include "util/string.h"
#include "nb_vector.h"

class Side;
class TextureCoord;

class Material
{
public:
    Material(const String &name);

    float a[4]; /* ambient color */
    float d[4]; /* diffuse color */
    float s[4]; /* specular color */
    float e[4]; /* emission color */
    float h[1]; /* specular exponent */

    short fl;   /* material flags */

    String f;   /* texture file name */
};

class Vertex
{
public:
    Vertex(const Vector3 &p, Side *s0, Side *s1, Side *s2);
    void move(const Vector3 &p);

    Vector3 p;  /* vertex position */
    Side *s0, *s1, *s2;
};

class Edge
{
public:
    Edge(Vertex *i, Vertex *j);

    Vertex *i, *j;
};

class Side
{
public:
    Side(const Vector3 &p0, const Vector3 &p1, const Vector3 &p2,
	 short tu, short tv, short r,
	 scalar su, scalar sv, const String &s);

    void move(const Vector3 &p);
    bool fore_side(const Vector3 &p) const;
    bool on_side(const Vector3 &p) const;

    Vector3 n;  /* plane normal vector */
    scalar d;   /* distance from origin */

    Vector3 plane_p;
    Vector3 plane_u;
    Vector3 plane_v;
    Material *plane_m;

    List<Vertex *> vertices;
    List<TextureCoord *> textureCoords;
};

class TextureCoord
{
public:
    TextureCoord(scalar u0, scalar u1);

    scalar u[2]; /* texture coordinate */
};

class Geometry
{
public:
    Geometry(TextureCoord *ti, Side *si, Vertex *vi,
	     TextureCoord *tj, Side *sj, Vertex *vj,
	     TextureCoord *tk, Side *sk, Vertex *vk,
	     Material *m);

    Material *m;
    TextureCoord *ti, *tj, *tk;
    Side *si, *sj, *sk;
    Vertex *vi, *vj, *vk;
};

#endif
