/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#include "nb_majormode.h"
#include "painter.h"
#include "hub.h"
#include "nb_object.h"
#include "nb_action.h"
#include "icons.h"
#include "drawing.h"
#include "nb_attribute.h"


PathMajorMode::PathMajorMode(Hub &hub) : MajorMode(hub),
					 startMinorMode(hub, *this),
					 connectMinorMode(hub, *this)
{
/** checkBox = new QCheckBox(_("Connect to existing path"));
    checkBox->setChecked(true);

    QGroupBox *groupBox = new QGroupBox(_("First point"));
    QVBoxLayout *l = new QVBoxLayout(groupBox);
    l->addWidget(startMinorMode.widget());

    QVBoxLayout *layout = new QVBoxLayout(widget());
    layout->addWidget(checkBox);
    layout->addWidget(groupBox);
    layout->addStretch(); **/

    setCaption(_("Path"));

    setMinorMode(&startMinorMode);
}

void PathMajorMode::paint(Painter &painter)
{
    MajorMode::paint(painter);

    painter.setPen(0x7f, 0x7f, 0x7f);
    if (minorMode() == &connectMinorMode && hub.pointerActive())
	painter.paintLine(startMinorMode.point(), connectMinorMode.point());

    painter.setBlackPen();
    if (minorMode() == &connectMinorMode || hub.pointerActive())
	painter.paintPoint(startMinorMode.point());
    if (minorMode() == &connectMinorMode && hub.pointerActive())
	painter.paintPoint(connectMinorMode.point());
}

List<ActionLine *> PathMajorMode::lines(ActionPoint *handle)
{
    /* this should be called somewhere */
    return List<ActionLine *>();
}

void PathMajorMode::cancel()
{
    setMinorMode(&startMinorMode, true);
}

void PathMajorMode::accept(MinorMode *minor_mode)
{
    if (minor_mode == &startMinorMode) {
	path = 0;
	setMinorMode(&connectMinorMode);
    }

    if (minor_mode == &connectMinorMode) {
	PathObject *path0, *path1;
	List<Action *> actions;

	if (path)
	    path0 = path->path();
	else {
	    path0 = new PathObject(startMinorMode.point(), hub.drawing());
	    actions << new NewObjectAction(path0);
	}

	path1 = new PathObject(connectMinorMode.point(), hub.drawing());
	actions << new NewObjectAction(path1);

	actions << new ConnectPathAction(path0, path1, hub.drawing());

	hub.drawing()->performActions(actions, icons::add, _("New path"));

	if (connectMinorMode.released)
	    cancel();
	else {
	    startMinorMode.setPoint(connectMinorMode.point());
	    path = path0;
	    setMinorMode(&connectMinorMode);
	}
    }
}

/****************************************************************************/

BodyMajorMode::BodyMajorMode(Hub &hub) : MajorMode(hub), minorMode(hub, *this)
{
/** QGroupBox *groupBox = new QGroupBox(_("Position"));
    QVBoxLayout *l = new QVBoxLayout(groupBox);
    l->addWidget(minorMode.widget());

    QVBoxLayout *layout = new QVBoxLayout(widget());
    layout->addWidget(groupBox);
    layout->addStretch(); **/

    setCaption(_("Body"));

    setMinorMode(&minorMode);
}

void BodyMajorMode::paint(Painter &painter)
{
    MajorMode::paint(painter);

    if (hub.pointerActive()) {
	painter.setBlackPen();
	painter.paintPoint(minorMode.point());
    }
}

List<ActionLine *> BodyMajorMode::lines(ActionPoint *handle)
{
    return List<ActionLine *>();
}

void BodyMajorMode::cancel()
{
    setMinorMode(&minorMode, true);
}

void BodyMajorMode::accept(MinorMode *minor_mode)
{
    hub.drawing()->performActions(
	List<Action *>() << new NewObjectAction(
	    new BodyObject(minorMode.point(), hub.drawing())),
	icons::add, _("New body"));
}

/****************************************************************************/

CoinMajorMode::CoinMajorMode(Hub &hub) : MajorMode(hub), minorMode(hub, *this)
{
/** checkBox = new QCheckBox(_("Place coin above the selected point"));

    QGroupBox *groupBox = new QGroupBox(_("Position"));
    QVBoxLayout *l = new QVBoxLayout(groupBox);
    l->addWidget(minorMode.widget());

    QVBoxLayout *layout = new QVBoxLayout(widget());
    layout->addWidget(checkBox);
    layout->addWidget(groupBox);
    layout->addStretch(); **/

    setCaption(_("Coin"));
    // setIcon(icons::neverball_coin_yellow);

    setMinorMode(&minorMode);
}

void CoinMajorMode::paint(Painter &painter)
{
    MajorMode::paint(painter);

    if (hub.pointerActive()) {
	painter.setBlackPen();
	painter.paintPoint(minorMode.point());
    }
}

List<ActionLine *> CoinMajorMode::lines(ActionPoint *handle)
{
    return List<ActionLine *>();
}

void CoinMajorMode::cancel()
{
    setMinorMode(&minorMode, true);
}

void CoinMajorMode::accept(MinorMode *minor_mode)
{
    Metrum3 pos = minorMode.point();
    if (/** checkBox->isChecked() **/ true)
	pos += Metrum3(0, 0, 16);

    hub.drawing()->performActions(
	List<Action *>() << new NewObjectAction(
	    new CoinObject(pos, hub.drawing())),
	icons::add, _("New coin"));
}

/****************************************************************************/

GoalMajorMode::GoalMajorMode(Hub &hub) : MajorMode(hub),
					 centerMinorMode(hub, *this),
					 perimeterMinorMode(hub, *this)
{
/** comboBox = new QComboBox;
    comboBox->addItem(_("Specify center only"));
    comboBox->addItem(_("Specify center and radius"));

    QGroupBox *groupBox = new QGroupBox(_("Center"));
    QVBoxLayout *l = new QVBoxLayout(groupBox);
    l->addWidget(centerMinorMode.widget());

    QVBoxLayout *layout = new QVBoxLayout(widget());
    layout->addWidget(comboBox);
    layout->addWidget(groupBox);
    layout->addStretch(); **/

    setCaption(_("Goal"));
    // setIcon(icons::neverball_goal);

    setMinorMode(&centerMinorMode);
}

void GoalMajorMode::paint(Painter &painter)
{
    MajorMode::paint(painter);

    if (minorMode() == &perimeterMinorMode && hub.pointerActive()) {
	Metrum3 delta = perimeterMinorMode.point() - centerMinorMode.point();
	delta.rz() = 0;
	painter.setPen(0x7f, 0x7f, 0x7f);
	painter.paintCircle(centerMinorMode.point(), delta.length());
	painter.paintLine(centerMinorMode.point(), perimeterMinorMode.point());
    }

    painter.setBlackPen();
    if (minorMode() == &perimeterMinorMode || hub.pointerActive())
	painter.paintPoint(centerMinorMode.point());
    if (minorMode() == &perimeterMinorMode && hub.pointerActive())
	painter.paintPoint(perimeterMinorMode.point());
}

List<ActionLine *> GoalMajorMode::lines(ActionPoint */*handle*/)
{
    return List<ActionLine *>();
}

void GoalMajorMode::accept(MinorMode *minor_mode)
{
    if (minor_mode == &centerMinorMode) {
	if (/** comboBox->currentIndex() **/ 1 == 1) {
	    setMinorMode(&perimeterMinorMode);
	    return;
	}

	hub.drawing()->performActions(
	    List<Action *>() << new NewObjectAction(
		new GoalObject(centerMinorMode.point(), hub.drawing())),
	    icons::add, _("New goal"));
	cancel();
    }

    if (minor_mode == &perimeterMinorMode) {
	Metrum3 delta = perimeterMinorMode.point() - centerMinorMode.point();
	delta.rz() = 0;
	GoalObject *goal = new GoalObject(centerMinorMode.point(),
					  hub.drawing());
	goal->setAttribute(Variant::fromMetrum(delta.length()),
			   &NeverballAttributes::radius);

	hub.drawing()->performActions(
	    List<Action *>() << new NewObjectAction(goal),
	    icons::add, _("New goal"));

	cancel();
    }
}

void GoalMajorMode::cancel()
{
    setMinorMode(&centerMinorMode, true);
}

/****************************************************************************/

JumpMajorMode::JumpMajorMode(Hub &hub) : MajorMode(hub),
					 posMinorMode(hub, *this),
					 radiusMinorMode(hub, *this),
					 targetMinorMode(hub, *this)
{
/** comboBox = new QComboBox;
    comboBox->addItem(_("Use fixed radius"));
    comboBox->addItem(_("Set radius interactively"));

    QGroupBox *groupBox = new QGroupBox(_("Center"));
    QVBoxLayout *l = new QVBoxLayout(groupBox);
    l->addWidget(posMinorMode.widget());

    QVBoxLayout *layout = new QVBoxLayout(widget());
    layout->addWidget(comboBox);
    layout->addWidget(groupBox);
    layout->addStretch(); **/

    setCaption(_("Jump"));
    // setIcon(icons::neverball_jump);

    setMinorMode(&posMinorMode);
}

void JumpMajorMode::paint(Painter &painter)
{
    MajorMode::paint(painter);

    if (minorMode() != &posMinorMode && hub.pointerActive()) {
	Metrum3 delta = radiusMinorMode.point() - posMinorMode.point();
	delta.rz() = 0;
	painter.setPen(0x7f, 0x7f, 0x7f);
	painter.paintCircle(posMinorMode.point(), delta.length());
	painter.paintLine(posMinorMode.point(), radiusMinorMode.point());

	if (minorMode() == &targetMinorMode)
	    painter.paintLine(posMinorMode.point(), targetMinorMode.point());
    }

    painter.setBlackPen();
    if (hub.pointerActive() || minorMode() != &posMinorMode)
	painter.paintPoint(posMinorMode.point());
    if ((minorMode() == &radiusMinorMode && hub.pointerActive()) ||
	minorMode() == &targetMinorMode)
	painter.paintPoint(radiusMinorMode.point());
    if (minorMode() == &targetMinorMode && hub.pointerActive())
	painter.paintPoint(targetMinorMode.point());
}

List<ActionLine *> JumpMajorMode::lines(ActionPoint *handle)
{
    return List<ActionLine *>();
}

void JumpMajorMode::cancel()
{
    setMinorMode(&posMinorMode, true);
/** comboBox->setEnabled(true); **/
}

void JumpMajorMode::accept(MinorMode *minor_mode)
{
    if (minor_mode == &posMinorMode) {
	if (/** comboBox->currentIndex() **/ 1 == 1)
	    setMinorMode(&radiusMinorMode);
	else {
	    radiusMinorMode.setPoint(posMinorMode.point() +
				     hub.drawing()->scale(0.5, 0.0, 0.0));
	    setMinorMode(&targetMinorMode);
	}
    /** comboBox->setEnabled(false); **/
    }

    if (minor_mode == &radiusMinorMode)
	setMinorMode(&targetMinorMode);

    if (minor_mode == &targetMinorMode) {
	JumpObject *jump = new JumpObject(posMinorMode.point(),
					  targetMinorMode.point(),
					  hub.drawing());

	if (/** comboBox->currentIndex() **/ 1 == 1) {
	    Metrum3 delta = radiusMinorMode.point() - posMinorMode.point();
	    delta.rz() = 0;
	    jump->setAttribute(Variant::fromMetrum(delta.length()),
			       &NeverballAttributes::radius);
	}

	hub.drawing()->performActions(
	    List<Action *>() << new NewObjectAction(jump),
	    icons::add, _("New jump"));

	cancel();
    }
}

/****************************************************************************/

SwitchMajorMode::SwitchMajorMode(Hub &hub) : MajorMode(hub),
					     posMinorMode(hub, *this),
					     radiusMinorMode(hub, *this),
					     connectMinorMode(hub, *this)

{
/** comboBox = new QComboBox;
    comboBox->addItem(_("Specify center only"));
    comboBox->addItem(_("Specify center and radius"));

    QGroupBox *groupBox = new QGroupBox(_("Center"));
    QVBoxLayout *l = new QVBoxLayout(groupBox);
    l->addWidget(posMinorMode.widget());

    QVBoxLayout *layout = new QVBoxLayout(widget());
    layout->addWidget(comboBox);
    layout->addWidget(groupBox);
    layout->addStretch(); **/

    setCaption(_("Switch"));
    // setIcon(icons::neverball_switch);

    setMinorMode(&posMinorMode);
}

void SwitchMajorMode::paint(Painter &painter)
{
    MajorMode::paint(painter);

    if (minorMode() != &posMinorMode && hub.pointerActive()) {
	Metrum3 delta = radiusMinorMode.point() - posMinorMode.point();
	delta.rz() = 0;
	painter.setPen(0x7f, 0x7f, 0x7f);
	painter.paintCircle(posMinorMode.point(), delta.length());
	painter.paintLine(posMinorMode.point(), radiusMinorMode.point());

	if (minorMode() == &connectMinorMode)
	    painter.paintLine(posMinorMode.point(), connectMinorMode.point());
    }

    painter.setBlackPen();
    if (hub.pointerActive() || minorMode() != &posMinorMode)
	painter.paintPoint(posMinorMode.point());
    if ((minorMode() == &radiusMinorMode && hub.pointerActive()) ||
	minorMode() == &connectMinorMode)
	painter.paintPoint(radiusMinorMode.point());
    if (minorMode() == &connectMinorMode && hub.pointerActive())
	painter.paintPoint(connectMinorMode.point());
}

List<ActionLine *> SwitchMajorMode::lines(ActionPoint *handle)
{
    return List<ActionLine *>();
}

void SwitchMajorMode::cancel()
{
    setMinorMode(&posMinorMode, true);
/** comboBox->setEnabled(true); **/
}

void SwitchMajorMode::accept(MinorMode *minor_mode)
{
    if (minor_mode == &posMinorMode) {
	if (/** comboBox->currentIndex() **/ 1 == 1)
	    setMinorMode(&radiusMinorMode);
	else {
	    radiusMinorMode.setPoint(posMinorMode.point() +
				     hub.drawing()->scale(0.5, 0.0, 0.0));
	    setMinorMode(&connectMinorMode);
	}
    /** comboBox->setEnabled(false); **/
    }

    if (minor_mode == &radiusMinorMode)
	setMinorMode(&connectMinorMode);

    if (minor_mode == &connectMinorMode) {
	SwitchObject *swch = new SwitchObject(posMinorMode.point(),
					      hub.drawing());

	if (/** comboBox->currentIndex() **/ 1 == 1) {
	    Metrum3 delta = radiusMinorMode.point() - posMinorMode.point();
	    delta.rz() = 0;
	    swch->setAttribute(Variant::fromMetrum(delta.length()),
			       &NeverballAttributes::radius);
	}

	/* TODO: set target */

	hub.drawing()->performActions(
	    List<Action *>() << new NewObjectAction(swch),
	    icons::add, _("New switch"));

	cancel();
    }
}

/****************************************************************************/

BillboardMajorMode::BillboardMajorMode(Hub &hub) : MajorMode(hub),
						   posMinorMode(hub, *this)
{
/** QGroupBox *groupBox = new QGroupBox(_("Center"));
    QVBoxLayout *l = new QVBoxLayout(groupBox);
    l->addWidget(posMinorMode.widget());

    QVBoxLayout *layout = new QVBoxLayout(widget());
    layout->addWidget(groupBox);
    layout->addStretch(); **/

    setCaption(_("Billboard"));

    setMinorMode(&posMinorMode);
}

void BillboardMajorMode::paint(Painter &painter)
{
}

List<ActionLine *> BillboardMajorMode::lines(ActionPoint *handle)
{
    return List<ActionLine *>();
}

void BillboardMajorMode::cancel()
{
    setMinorMode(&posMinorMode, true);
}

void BillboardMajorMode::accept(MinorMode *minor_mode)
{
    hub.drawing()->performActions(
	List<Action *>() << new NewObjectAction(
	    new BillboardObject(posMinorMode.point(), hub.drawing())),
	icons::add, _("New billboard"));
}

/****************************************************************************/

UserMajorMode::UserMajorMode(Hub &hub) : MajorMode(hub),
					 posMinorMode(hub, *this),
					 radiusMinorMode(hub, *this)
{
/** radiusCheckBox = new QCheckBox(_("Interactively select radius"));
    radiusCheckBox->setChecked(false);

    offsetCheckBox = new QCheckBox(_("Place above selected position"));
    offsetCheckBox->setChecked(true);

    QGroupBox *groupBox = new QGroupBox(_("Center"));
    QVBoxLayout *l = new QVBoxLayout(groupBox);
    l->addWidget(posMinorMode.widget());

    QVBoxLayout *layout = new QVBoxLayout(widget());
    layout->addWidget(radiusCheckBox);
    layout->addWidget(offsetCheckBox);
    layout->addWidget(groupBox);
    layout->addStretch(); **/

    setCaption(_("User"));

    setMinorMode(&posMinorMode);
}

void UserMajorMode::paint(Painter &painter)
{
    MajorMode::paint(painter);

    if (minorMode() == &radiusMinorMode && hub.pointerActive()) {
	Metrum3 delta = radiusMinorMode.point() - posMinorMode.point();
	delta.rz() = 0;
	painter.setPen(0x7f, 0x7f, 0x7f);
	painter.paintCircle(posMinorMode.point(), delta.length());
	painter.paintLine(posMinorMode.point(), radiusMinorMode.point());
    }

    painter.setBlackPen();
    if (minorMode() == &radiusMinorMode || hub.pointerActive())
	painter.paintPoint(posMinorMode.point());
    if (minorMode() == &radiusMinorMode && hub.pointerActive())
	painter.paintPoint(radiusMinorMode.point());
}

List<ActionLine *> UserMajorMode::lines(ActionPoint *handle)
{
    return List<ActionLine *>();
}

void UserMajorMode::cancel()
{
    setMinorMode(&posMinorMode, true);
}

void UserMajorMode::accept(MinorMode *minor_mode)
{
    if (minor_mode == &posMinorMode) {
	if (/** radiusCheckBox->isChecked() **/ true) {
	    setMinorMode(&radiusMinorMode);
	    return;
	}

	hub.drawing()->performActions(
	    List<Action *>() << new NewObjectAction(
		new UserObject(posMinorMode.point(), hub.drawing())),
	    icons::add, _("New user"));
	cancel();
    }

    if (minor_mode == &radiusMinorMode) {
	Metrum3 delta = radiusMinorMode.point() - posMinorMode.point();
	delta.rz() = 0;
	UserObject *user = new UserObject(posMinorMode.point(),
					  hub.drawing());
	user->setAttribute(Variant::fromMetrum(delta.length()),
			   &NeverballAttributes::radius);

	hub.drawing()->performActions(
	    List<Action *>() << new NewObjectAction(user),
	    icons::add, _("New user"));

	cancel();
    }
}

/****************************************************************************/

ViewMajorMode::ViewMajorMode(Hub &hub) : MajorMode(hub),
					 posMinorMode(hub, *this),
					 targetMinorMode(hub, *this)
{
/** QGroupBox *groupBox = new QGroupBox(_("First point"));
    QVBoxLayout *l = new QVBoxLayout(groupBox);
    l->addWidget(posMinorMode.widget());

    QVBoxLayout *layout = new QVBoxLayout(widget());
    layout->addWidget(groupBox);
    layout->addStretch(); **/

    setCaption(_("View"));
    setIcon(icons::eye);

    setMinorMode(&posMinorMode);
}

void ViewMajorMode::paint(Painter &painter)
{
    MajorMode::paint(painter);

    painter.setPen(0x7f, 0x7f, 0x7f);
    if (minorMode() == &targetMinorMode && hub.pointerActive())
	painter.paintLine(posMinorMode.point(), targetMinorMode.point());

    painter.setBlackPen();
    if (minorMode() == &targetMinorMode || hub.pointerActive())
	painter.paintPoint(posMinorMode.point());
    if (minorMode() == &targetMinorMode && hub.pointerActive())
	painter.paintPoint(targetMinorMode.point());
}

List<ActionLine *> ViewMajorMode::lines(ActionPoint *handle)
{
    return List<ActionLine *>();
}

void ViewMajorMode::cancel()
{
    setMinorMode(&posMinorMode, true);
}

void ViewMajorMode::accept(MinorMode *minor_mode)
{
    if (minor_mode == &posMinorMode)
	setMinorMode(&targetMinorMode);

    if (minor_mode == &targetMinorMode) {
	hub.drawing()->performActions(
	    List<Action *>() << new NewObjectAction(
		new ViewObject(posMinorMode.point(),
			       targetMinorMode.point(), hub.drawing())),
	    icons::add, _("New view"));

	cancel();
    }
}
