/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#include "nb_readxml.h"
#include "xml.h"
#include "nb_object.h"
#include "dirmodel.h"


class NeverballXMLHandler : public XMLHandler
{
public:
    NeverballXMLHandler(Drawing *drawing, ObjectContext *context);
    void element(XMLElement *element, Messages &messages);

private:
    Drawing *const drawing;
    ObjectContext *const context;
};

NeverballXMLHandler::NeverballXMLHandler(Drawing *drawing,
					 ObjectContext *context)
    : drawing(drawing), context(context)
{
}

void NeverballXMLHandler::element(XMLElement *element, Messages &messages)
{
    if (element->name() == "path")
	context->addObject(new PathObject(element, drawing));
    else if (element->name() == "body")
	context->addObject(new BodyObject(element, drawing));
    else if (element->name() == "coin")
	context->addObject(new CoinObject(element, drawing));
    else if (element->name() == "goal")
	context->addObject(new GoalObject(element, drawing));
    else if (element->name() == "jump")
	context->addObject(new JumpObject(element, drawing));
    else if (element->name() == "switch")
	context->addObject(new SwitchObject(element, drawing));
    else if (element->name() == "billboard")
	context->addObject(new BillboardObject(element, drawing));
    else if (element->name() == "ball")
	context->addObject(new UserObject(element, drawing));
    else if (element->name() == "view")
	context->addObject(new ViewObject(element, drawing));
    else
	XMLHandler::element(element, messages);
}

DrawingItem *neverball_read_xml(XMLElement *element, Messages &messages,
				Library *library, FileItem *fileItem)
{
    DrawingItem *item = new DrawingItem(Drawing::NeverballFile,
					library, fileItem);
    Drawing *drawing = item->drawing();

    element->setHandler(new NeverballXMLHandler(drawing,
						drawing->rootContext()));

    drawing->setBaseBase(16);
    drawing->setBaseExponent(2);

    if (element->selectAttribute("text"))
	drawing->setMapText(element->value());

    return item;
}
