/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef NB_VECTOR_H
#define NB_VECTOR_H

#include <math.h>

typedef float scalar;

class Vector3
{
public:
    Vector3() : xp(0.f), yp(0.f), zp(0.f) {
    }
    Vector3(scalar x, scalar y, scalar z) : xp(x), yp(y), zp(z) {
    }

    scalar x() const {
	return xp;
    }
    scalar y() const {
	return yp;
    }
    scalar z() const {
	return zp;
    }

    Vector3 &operator+=(const Vector3 &p) {
	xp += p.xp;
	yp += p.yp;
	zp += p.zp;
	return *this;
    }
    Vector3 &operator-=(const Vector3 &p) {
	xp -= p.xp;
	yp -= p.yp;
	zp -= p.zp;
	return *this;
    }
    Vector3 &operator*=(scalar c) {
	xp *= c;
	yp *= c;
	zp *= c;
	return *this;
    }
    Vector3 &operator/=(scalar c) {
	xp /= c;
	yp /= c;
	zp /= c;
	return *this;
    }

    Vector3 operator+(const Vector3 &p) const {
	return Vector3(xp + p.xp, yp + p.yp, zp + p.zp);
    }
    Vector3 operator-(const Vector3 &p) const {
	return Vector3(xp - p.xp, yp - p.yp, zp - p.zp);
    }
    Vector3 operator*(scalar c) const {
	return Vector3(xp * c, yp * c, zp * c);
    }
    Vector3 operator/(scalar c) const {
	return Vector3(xp / c, yp / c, zp / c);
    }
    Vector3 operator-() const {
	return Vector3(-xp, -yp, -zp);
    }
private:
    scalar xp;
    scalar yp;
    scalar zp;
};

inline Vector3 operator*(scalar c, const Vector3 &p)
{
    return p * c;
}

inline scalar v_dot(const Vector3 &u, const Vector3 &v)
{
    return u.x() * v.x() + u.y() * v.y() + u.z() * v.z();
}

inline scalar v_len(const Vector3 &u)
{
    return sqrt((double) v_dot(u, u));
}

inline Vector3 v_nrm(const Vector3 &v)
{
    return v / v_len(v);
}

inline Vector3 v_crs(const Vector3 &v, const Vector3 &w)
{
    return Vector3(v.y() * w.z() - v.z() * w.y(),
		   v.z() * w.x() - v.x() * w.z(),
		   v.x() * w.y() - v.y() * w.x());
}

#endif
