/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef OBJECT_H
#define OBJECT_H

#include "util/metrum.h"
#include "util/list.h"
#include "util/screen.h"

class Metrum3;
class Drawing;
class ObjectContext;
class ObjectItem;
class String;
class Icon;
class Painter;
class ActionPoint;
class ActionLine;
class View;
class XMLWriter;
class Attribute;
class Variant;
class ObjectInterior;
class ItemInterior;

class EmbeddingLayer
{
public:
    virtual int angle() const;
    virtual bool mirror() const;
    virtual Metrum transform(Metrum m) const;
    virtual Metrum3 transform(const Metrum3 &pos) const;
};

class Embedding : public List<EmbeddingLayer *>
{
public:
    Embedding();
    Embedding(const Embedding &embedding, EmbeddingLayer *l);

    int angle() const;
    bool mirror() const;
    Metrum transform(Metrum m) const;
    Metrum3 transform(const Metrum3 &pos) const;
};

class Object
{
public:
    Object(Drawing *drawing);
    virtual ~Object();
    Drawing *drawing() const;
    ObjectContext *context() const;
    void setContext(ObjectContext *context);
    Object *parent() const;
    ObjectItem *item() const;

    virtual String name() const = 0;
    virtual Icon *icon() const;
    virtual ObjectContext *children() const;
    virtual void paint(Painter &painter,
		       const Embedding &embedding = Embedding());
    virtual void paintShadow(const Metrum3 &pos, const ActionPoint *handle,
			     Painter &painter);

    virtual List<ActionPoint *> handles();
    virtual Metrum3 handlePos(const ActionPoint *handle) const;
    virtual List<ActionLine *> handleLines(const ActionPoint *handle);
    virtual void moveHandle(const Metrum3 &pos, const ActionPoint *handle);
    virtual Metrum3 lineBase(const ActionLine *line) const;
    virtual Metrum3 lineTangent(const ActionLine *line) const;
    virtual ScreenDist proximity(const ScreenPos &point, View *view,
				 const Embedding &embedding = Embedding())
	const = 0;
    virtual bool within(const ScreenRect &rect, View *view,
			bool partially = false,
			const Embedding &embedding = Embedding()) const = 0;
    virtual void write(XMLWriter *writer) = 0;
    virtual List<Attribute *> attributes() const = 0;
    virtual Variant attribute(const Attribute *attribute) const = 0;
    virtual Variant attributeDefault(const Attribute *attribute) const = 0;
    virtual void setAttribute(const Variant &value,
			      const Attribute *attribute) = 0;

    virtual Metrum3 relativePos() const;
    Metrum3 absolutePos() const;

private:
    Drawing *const the_drawing;
    ObjectContext *the_context;
    ObjectItem *_item;
    void construct_item();
};

class ObjectContext
{
    Drawing *const _drawing;
    Object *const _parent;
    List<Object *> _objects;
    ObjectInterior *const _interior;
public:
    ObjectContext(Object *parent);
    ObjectContext(Drawing *drawing);
    ~ObjectContext();
    Drawing *drawing() const;
    Object *parent() const;
    ItemInterior *interior() const;

    List<Object *> objects() const;
    void addObject(Object *object);
    void insertObject(int index, Object *object);
    void removeObject(Object *object);
    void write(XMLWriter *writer);

    Metrum3 origin() const;
};

#endif
