/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef PAINTER_H
#define PAINTER_H

#include "util/metrum.h"

class Hub;
class Object;
class Metrum3;
class ScreenRect;
class ScreenPos;

class Painter
{
public:
    Painter(Hub &hub);
    bool isSelected(Object *object) const;

    virtual void setBlackPen() = 0;
    virtual void setWhitePen() = 0;
    virtual void setPen(unsigned int red,
			unsigned int green,
			unsigned int blue) = 0;
    virtual void setDashedBlackPen(char dashes, int dash_offset) = 0;
    virtual void setDashedPen(char dashes, int dash_offset,
			      unsigned int red,
			      unsigned int green,
			      unsigned int blue) = 0;

    void paintPoint(const Metrum3 &point);
    void paintActivePoint(const Metrum3 &point);
    void paintHandle(const Metrum3 &point,
		     unsigned int red, unsigned int green, unsigned int blue);
    void paintCircle(const Metrum3 &point, Metrum radius);
    void paintLine(const Metrum3 &point0, const Metrum3 &point1,
		   bool extend = false);
    void paintRectangle(const ScreenRect &rect,
	unsigned int red, unsigned int green, unsigned int blue);

protected:
    Hub &hub;
    virtual ScreenRect visibleRect() const = 0;
    virtual void drawLine(int x0, int y0, int x1, int y1) = 0;
    virtual void drawLine(const ScreenPos &p0, const ScreenPos &p1) = 0;
    virtual void drawRectangle(int x, int y,
			       unsigned int width, unsigned int height) = 0;
    virtual void fillRectangle(int x, int y,
			       unsigned int width, unsigned int height) = 0;
    virtual void drawArc(int x, int y, unsigned int width, unsigned int height,
			 int angle1 = 0, int angle2 = 0x5a00) = 0;
};

#endif
