/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef SCH_COMPONENT_H
#define SCH_COMPONENT_H

#include "majormode.h"
#include "object.h"
#include "action_pl.h"

class ComponentMajorMode : public MajorMode
{
public:
    ComponentMajorMode(Hub &hub);
    void paint(Painter &painter);
    List<ActionLine *> lines(ActionPoint *handle);

    void accept(MinorMode *minor_mode);
    void cancel();

private:
    PointSelectMinorMode minorMode;
/** QComboBox *comboBox; **/
};

class ComponentObject : public Object
{
public:
    ComponentObject(const Metrum3 &pos, Drawing *drawing);
    Metrum3 relativePos() const;
    String name() const;
    ObjectContext *children() const;
    void paint(Painter &painter, const Embedding &embedding);
    void paintShadow(const Metrum3 &pos, const ActionPoint *handle,
		     Painter &painter);

    List<ActionPoint *> handles();
    Metrum3 handlePos(const ActionPoint *handle) const;
    List<ActionLine *> handleLines(const ActionPoint *handle);
    void moveHandle(const Metrum3 &pos, const ActionPoint *handle);
    Metrum3 lineBase(const ActionLine *line) const;
    Metrum3 lineTangent(const ActionLine *line) const;
    ScreenDist proximity(const ScreenPos &point, View *view,
			 const Embedding &embedding) const;
    bool within(const ScreenRect &rect, View *view, bool partially,
		const Embedding &embedding) const;
    void write(XMLWriter *writer);
    List<Attribute *> attributes() const;
    Variant attribute(const Attribute *attribute) const;
    Variant attributeDefault(const Attribute *attribute) const;
    void setAttribute(const Variant &value, const Attribute *attribute);

    Metrum3 pos;
    bool selectable;
    int angle;
    bool mirror;
    String basename;

    Drawing *contents;
    bool is_embedded;
    ObjectContext context;

private:
    ActionPoint handle;
};

#endif
