/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef UTIL_SCREEN_H
#define UTIL_SCREEN_H

typedef double ScreenDist;

class ScreenPos
{
public:
    ScreenPos() : xp(0.0), yp(0.0) {
    }
    ScreenPos(ScreenDist x, ScreenDist y) : xp(x), yp(y) {
    }
    ScreenDist x() const {
	return xp;
    }
    ScreenDist y() const {
	return yp;
    }
    ScreenPos operator+(const ScreenPos &p) const {
	return ScreenPos(xp + p.xp, yp + p.yp);
    }
    ScreenPos operator-(const ScreenPos &p) const {
	return ScreenPos(xp - p.xp, yp - p.yp);
    }
    ScreenPos operator*(double c) const {
	return ScreenPos(xp * c, yp * c);
    }
    ScreenPos operator/(double c) const {
	return ScreenPos(xp / c, yp / c);
    }
    void setX(ScreenDist x) {
	xp = x;
    }
    void setY(ScreenDist y) {
	yp = y;
    }
    friend inline bool operator==(const ScreenPos &p0, const ScreenPos &p1);

    ScreenDist length() const;
    void computeParameters(const ScreenPos &base, const ScreenPos &tangent,
			   ScreenDist &parameter, ScreenDist &distance) const;
private:
    ScreenDist xp, yp;
};

inline bool operator==(const ScreenPos &p0, const ScreenPos &p1)
{
    return p0.xp == p1.xp && p0.yp == p1.yp;
}

inline ScreenPos operator*(double c, const ScreenPos &p) {
    return p * c;
}

class ScreenRect
{
    ScreenDist _x0, _y0, _x1, _y1;
public:
    ScreenRect() : _x0(0.), _y0(0.), _x1(0.), _y1(0.) {
    }
    ScreenRect(const ScreenPos &p0, const ScreenPos &p1)
      : _x0(p0.x() > p1.x() ? p1.x() : p0.x()),
	_y0(p0.y() > p1.y() ? p1.y() : p0.y()),
	_x1(p0.x() > p1.x() ? p0.x() : p1.x()),
	_y1(p0.y() > p1.y() ? p0.y() : p1.y()) {
    }
    ScreenDist x0() const { return _x0; }
    ScreenDist y0() const { return _y0; }
    ScreenDist x1() const { return _x1; }
    ScreenDist y1() const { return _y1; }

    bool contains(const ScreenPos &p) const {
	return _x0 != _x1 && p.x() >= _x0 && p.x() <= _x1 &&
	       _y0 != _y1 && p.y() >= _y0 && p.y() <= _y1;
    }
    bool contains(const ScreenRect &r) const {
	return _x0 != _x1 && r._x0 != r._x1 && r._x0 >= _x0 && r._x1 <= _x1 &&
	       _y0 != _y1 && r._y0 != r._y1 && r._y0 >= _y0 && r._y1 <= _y1;
    }
};

#endif
