/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#include "variant.h"


Variant::Variant(ValueType type, String const &s, int64_t i,
				 float f, double d, void *ptr)
    : type(type), s(s), i(i), f(f), d(d), ptr(ptr)
{
}

Variant::Variant() : type(vNull), i(0), f(0.), d(0.), ptr(NULL)
{
}

bool Variant::operator==(Variant v)
{
    if (v.type != type)
	return false;
    switch (type) {
    case vNull: return true;
    case vString: return s == v.s;
    case vMetrum:
    case vInt:
    case vBool: return i == v.i;
    case vFloat: return f == v.f;
    case vDouble: return d == v.d;
    case vPointer: return ptr == v.ptr;
    }
    return false;
}

bool Variant::operator!=(Variant v)
{
    return !operator==(v);
}

Variant Variant::fromString(String const &s)
{
    return Variant(vString, s, 0, 0., 0., NULL);
}

Variant Variant::fromMetrum(Metrum const &m)
{
    return Variant(vMetrum, String(), m, 0., 0., NULL);
}

Variant Variant::fromInt(int i)
{
    return Variant(vInt, String(), i, 0., 0., NULL);
}

Variant Variant::fromBool(bool b)
{
    return Variant(vBool, String(), b, 0., 0., NULL);
}

Variant Variant::fromFloat(float f)
{
    return Variant(vFloat, String(), 0, f, 0., NULL);
}

Variant Variant::fromDouble(double d)
{
    return Variant(vDouble, String(), 0, 0., d, NULL);
}

bool Variant::isNull() const
{
    return type == vNull;
}

String Variant::toString() const
{
    assert(type == vString);
    return s;
}

Metrum Variant::toMetrum() const
{
    assert(type == vMetrum);
    return i;
}

int Variant::toInt() const
{
    assert(type == vInt);
    return i;
}

bool Variant::toBool() const
{
    assert(type == vBool);
    return i;
}

float Variant::toFloat() const
{
    assert(type == vFloat);
    return f;
}

double Variant::toDouble() const
{
    assert(type == vDouble);
    return d;
}
