/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef VIEW_H
#define VIEW_H

#include "util/screen.h"
#include "util/metrum3.h"

class Drawing;

class View
{
public:
    View(Drawing *drawing);
    Drawing *drawing() const;

    virtual ScreenPos map(const Metrum3 &) const = 0;
    virtual Metrum3 unmap(const ScreenPos &) const = 0;

    virtual void setViewportDimensions(ScreenDist width,
				       ScreenDist height) = 0;
    struct ScrollInformation {
	ScrollInformation();
	unsigned int horzRange, horzPageSize, horzPosition;
	unsigned int vertRange, vertPageSize, vertPosition;
    };
    virtual ScrollInformation getScrollInformation() const = 0;

    virtual View *zoom(double factor, const ScreenPos &center) const = 0;
    virtual View *move(const ScreenPos &delta) const = 0;
    virtual View *scrollTo(unsigned int x, unsigned int y) const = 0;

private:
    Drawing *const the_drawing;
};

class SimpleView : public View
{
public:
    SimpleView(const Metrum3 &origin, double scale, double rotate,
	       Drawing *drawing);

    Metrum3 origin() const;
    double scale() const;
    double rotate() const;

    ScreenPos map(const Metrum3 &) const;
    Metrum3 unmap(const ScreenPos &) const;

    void setViewportDimensions(ScreenDist width, ScreenDist height);
    ScrollInformation getScrollInformation() const;

    View *zoom(double factor, const ScreenPos &center) const;
    View *move(const ScreenPos &delta) const;
    View *scrollTo(unsigned int x, unsigned int y) const;

private:
    Metrum3 const the_origin;
    double const the_scale, the_rotate;
    double const r_cos, r_sin;

    Metrum x_size, x_start, x_stop;
    Metrum y_size, y_start, y_stop;
};

class OrthogonalView : public View
{
public:
    OrthogonalView(const Metrum3 &origin, const ScreenPos &x,
		   const ScreenPos &y, const ScreenPos &z, Drawing *drawing);

    Metrum3 origin() const;

    ScreenPos map(const Metrum3 &) const;
    Metrum3 unmap(const ScreenPos &) const;

    void setViewportDimensions(ScreenDist width, ScreenDist height);
    ScrollInformation getScrollInformation() const;

    View *zoom(double factor, const ScreenPos &center) const;
    View *move(const ScreenPos &delta) const;
    View *scrollTo(unsigned int x, unsigned int y) const;

private:
    Metrum3 const the_origin;
    ScreenPos const x, y, z;
};

#endif
