/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#include <stdio.h>
#include "actionsets.h"
#include "example.h"


FileActionSet::FileActionSet(Connection &conn, Example &example)
    : example(example), save_action(*this), print_action(*this),
      quit_action(*this), menu_builder(conn)
{
    save_action.setText("Save");
    save_action.setHotkey("C-s");

    print_action.setText("Print");
    print_action.setHotkey("C-p");

    quit_action.setText("Quit");
    quit_action.setHotkey("C-q");

    menu_builder.addAction(save_action);
    menu_builder.addAction(print_action);
    menu_builder.addSeparator();
    menu_builder.addAction(quit_action);
}

Menu *FileActionSet::menu() const
{
    return menu_builder.menu();
}

void FileActionSet::actionTriggered(Action *action)
{
    if (action == &save_action) {
	printf("Save\n");
	return;
    }
    if (action == &print_action) {
	printf("Print\n");
	return;
    }
    if (action == &quit_action) {
	printf("Quit\n");
	example.close();
	return;
    }
}

/****************************************************************************/

ViewActionSet::ViewActionSet(Connection &conn)
    : zoom_in_action(*this), zoom_out_action(*this), docks_action(*this),
      tab_bar_action(*this), docks_action_set(conn), menu_builder(conn)
{
    zoom_in_action.setText("Zoom in");
    zoom_in_action.setHotkey("z");

    zoom_out_action.setText("Zoom out");
    zoom_out_action.setHotkey("Z");

    docks_action.setText("Dock windows");
    docks_action.setMenu(docks_action_set.menu());

    tab_bar_action.setText("Show tab bar");

    menu_builder.addAction(zoom_in_action);
    menu_builder.addAction(zoom_out_action);
    menu_builder.addSeparator();
    menu_builder.addAction(docks_action);
    menu_builder.addAction(tab_bar_action);
}

Menu *ViewActionSet::menu() const
{
    return menu_builder.menu();
}

void ViewActionSet::actionTriggered(Action *action)
{
    printf("View menu action triggered: %s\n", action->text().c_str());
}

/****************************************************************************/

DocksActionSet::DocksActionSet(Connection &conn)
{
    Action *action;

    action = new Action(*this);
    action->setText("Project directories");
    actions.push_back(action);

    action = new Action(*this);
    action->setText("Objects");
    actions.push_back(action);

    action = new Action(*this);
    action->setText("Properties");
    actions.push_back(action);

    action = new Action(*this);
    action->setText("History");
    actions.push_back(action);

    action = new Action(*this);
    action->setText("Modes");
    actions.push_back(action);

    action = new Action(*this);
    action->setText("Library");
    actions.push_back(action);

    action = new Action(*this);
    action->setText("Input");
    actions.push_back(action);

    menu_builder = new MenuBuilder(conn);
    for (unsigned int i = 0; i < actions.size(); i++)
	menu_builder->addAction(*actions[i]);
}

DocksActionSet::~DocksActionSet()
{
    delete menu_builder;
    for (unsigned int i = 0; i < actions.size(); i++)
	delete actions[i];
}

Menu *DocksActionSet::menu() const
{
    return menu_builder->menu();
}

void DocksActionSet::actionTriggered(Action *action)
{
    printf("Docks menu action triggered: %s\n", action->text().c_str());
}

/****************************************************************************/

PopupActionSet::PopupActionSet(Connection &conn)
{
    Action *action;

    action = new Action(*this);
    action->setText("Net");
    action->setHotkey("n");
    actions.push_back(action);

    action = new Action(*this);
    action->setText("Bus");
    action->setHotkey("u");
    actions.push_back(action);

    action = new Action(*this);
    action->setText("Component");
    action->setHotkey("c");
    actions.push_back(action);

    actions.push_back(0);

    action = new Action(*this);
    action->setText("Line");
    action->setHotkey("l");
    actions.push_back(action);

    action = new Action(*this);
    action->setText("Picture");
    action->setHotkey("g");
    actions.push_back(action);

    action = new Action(*this);
    action->setText("Box");
    action->setHotkey("b");
    actions.push_back(action);

    action = new Action(*this);
    action->setText("Circle");
    action->setHotkey("v");
    actions.push_back(action);

    action = new Action(*this);
    action->setText("Arc");
    action->setHotkey("a");
    actions.push_back(action);

    action = new Action(*this);
    action->setText("Text");
    action->setHotkey("t");
    actions.push_back(action);

    action = new Action(*this);
    action->setText("Path");
    action->setHotkey("h");
    actions.push_back(action);

    actions.push_back(0);

    action = new Action(*this);
    action->setText("Select all");
    action->setHotkey("C-a");
    actions.push_back(action);

    actions.push_back(0);

    action = new Action(*this);
    action->setText("View");
    actions.push_back(action);

    menu_builder = new MenuBuilder(conn);
    for (unsigned int i = 0; i < actions.size(); i++)
	if (actions[i])
	    menu_builder->addAction(*actions[i]);
	else
	    menu_builder->addSeparator();
}

PopupActionSet::~PopupActionSet()
{
    delete menu_builder;
    for (unsigned int i = 0; i < actions.size(); i++)
	delete actions[i];
}

Menu *PopupActionSet::menu() const
{
    return menu_builder->menu();
}

void PopupActionSet::actionTriggered(Action *action)
{
    printf("Popup menu action triggered: %s\n", action->text().c_str());
}
