/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#include "example.h"
#include "icons.h"


ExampleWidgets::ExampleWidgets(Connection &conn, Menu *fileMenu,
						 Menu *viewMenu)
    : menu_bar_builder(conn),
      viewport(conn),
      navigator(&viewport),
      pane(conn),
      dock_manager(conn),
      status_bar(conn),
      root_widget(conn),

      properties_view(conn),
      properties_pane(conn),
      properties_dock(dock_manager, "Properties", icons::tree_leaf,
		      &properties_pane, "properties",
		      conn.screen().blackPixel(),
		      conn.colors().viewBackground.pixel),
      button0(conn),
      button1(conn),
      button_layout(conn),
      buttons_dock(dock_manager, "Buttons", icons::tree_leaf,
		   &button_layout, "buttons",
		   conn.screen().blackPixel(),
		   conn.colors().background.pixel)
{
    menu_bar_builder.addMenu("File", fileMenu);
    menu_bar_builder.addMenu("View", viewMenu);

    pane.mainControl()->setWidget(&viewport);
    pane.navigatorControl()->setWidget(&navigator);
    dock_manager.mainControl()->setWidget(&pane);
    dock_manager.setConfiguration(
	"(((buttons) 1.0) 0.25) (((properties) 1.0) 0.25) () () ()");

    root_widget.addWidget(menu_bar_builder.menuBar());
    root_widget.addWidget(dock_manager.widget());
    root_widget.addWidget(&status_bar);

    properties_view.setRootInterior(&attribute_model.interior);
    properties_pane.mainControl()->setWidget(&properties_view);

    button0.setText("Click me!");
    button1.setText("Click me!");

    button_layout.setBackground(conn.colors().background.pixel);
    button_layout.setPadding(8);
    button_layout.setSpacing(8);
    button_layout.addWidget(&button0);
    button_layout.addWidget(&button1);
}

Widget *ExampleWidgets::rootWidget()
{
    return &root_widget;
}

Focussable *ExampleWidgets::focusHandler()
{
    return dock_manager.focusHandler();
}

StatusBar *ExampleWidgets::statusBar()
{
    return &status_bar;
}

static X::Window make_window(Connection &conn, int x, int y,
			     unsigned int width, unsigned int height,
			     unsigned int border_width, int argc, char *argv[])
{
    X::Window window;
    Atom protocols[] = { conn.atoms().WM_DELETE_WINDOW };

    window = conn.screen().rootWindow().createWindow(
	x, y, width, height, border_width, conn.visualInfo().depth,
	InputOutput, conn.visual(), 0, NULL);

    window.utf8SetWMProperties("Toolkit Example", "Toolkit Example",
			       argv, argc, 0, 0, 0);

    window.setWMProtocols(protocols, 1);

    window.selectInput(KeyPressMask | KeyReleaseMask |
		       ButtonPressMask | ButtonReleaseMask |
		       EnterWindowMask | LeaveWindowMask |
		       PointerMotionMask | ExposureMask |
		       VisibilityChangeMask | StructureNotifyMask |
		       FocusChangeMask);
    window.map();

    return window;
}

/****************************************************************************/

Example::Example(Connection &conn, int argc, char **argv)
    : file_action_set(conn, *this),
      view_action_set(conn),
      widgets(conn, file_action_set.menu(), view_action_set.menu()),
      window(make_window(conn, 0, 0, 800, 600, 1, argc, argv)),
      hook(conn, window, this, 0, 0, 800, 600, 1)
{
    hook.control()->setWidget(widgets.rootWidget());
    hook.setFocusHandler(widgets.focusHandler());
    hook.setStatusBar(widgets.statusBar());
    hook.registerActionSet(&file_action_set);
    hook.registerActionSet(&view_action_set);
}

void Example::close()
{
    hook.close();
}
