/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef TOOLKIT_ACTION_H
#define TOOLKIT_ACTION_H

#include "hotkey.h"

class Icon;

namespace Toolkit {

class Action;
class Menu;

class ActionSet
{
    friend class Action;
    std::vector<Action *> _actions;
public:
    ~ActionSet();
    virtual void actionTriggered(Action *action);
    std::vector<Action *> actions() const;
};

class Action
{
    ActionSet &action_set;
    std::string _text;
    int _mnemonic;
    Hotkey _hotkey;
    Icon *_icon;
    Menu *_menu;
    bool is_active, is_disabled;
    std::string disable_message;
public:
    Action(ActionSet &actionSet);
    ~Action();
    std::string text() const;
    int mnemonic() const;
    void setText(const std::string &text);
    void setLiteralText(const std::string &text, int mnemonic = -1);
    Hotkey hotkey() const;
    void setHotkey(const Hotkey &hotkey);
    Icon *icon() const;
    void setIcon(Icon *icon);
    Menu *menu() const;
    void setMenu(Menu *menu);

    bool isActive() const;
    void setActive(bool active);
    bool isDisabled() const;
    std::string disableMessage() const;
    void enable();
    void disable(const std::string &message);

    class Observer {
    public:
	Observer(Action &action);
	~Observer();
	Action &action;
	virtual void actionChanged() = 0;
    };

    void trigger();
private:
    std::vector<Observer *> observers;
    void update_observers();
};

} /* namespace Toolkit */

#endif
