/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef TOOLKIT_BUTTONS_H
#define TOOLKIT_BUTTONS_H

#include "widget.h"

namespace Toolkit {

class ButtonicWidget : public Widget
{
    int decoration_width;
    int decoration_height;
    bool highlighted;
public:
    ButtonicWidget(Connection &conn);

    void paint(const Region &region, X::Pixmap pixmap);

    void enterEvent(Time time, int x, int y, int x_root, int y_root,
		    unsigned int state/*, char is_hint*/);
    void leaveEvent(Time time, int x, int y, int x_root, int y_root,
		    unsigned int state/*, char is_hint*/);
protected:
    class CornerRoundness
    {
    public:
	CornerRoundness();
	CornerRoundness(int r);
	CornerRoundness(int topLeft, int topRight,
			int bottomLeft, int bottomRight);
	int const topLeft, topRight;
	int const bottomLeft, bottomRight;
    };

    int decorationWidth() const;
    int decorationHeight() const;
    void setDecorationSize(int width, int height);
    void paintOuterBorder(unsigned long color, unsigned long pixelColor,
			  const Region &region, X::Pixmap pixmap);
    void paintInnerBorder(unsigned long color0, unsigned long color1,
			  const Region &region, X::Pixmap pixmap);

    virtual void paintDecoration(int xpos, int ypos, X::Pixmap pixmap) = 0;

private:
    virtual CornerRoundness cornerRoundness() const = 0;
    virtual void updateSizeRequirements();
};

class GenericButton : public ButtonicWidget
{
    bool pressed, _insensitive;
    virtual void press(Time time, int x, int y, int x_root, int y_root,
		       unsigned int state);
protected:
    bool insensitive() const;
    void setInsensitive(bool insensitive);
public:
    GenericButton(Connection &conn);
    void paint(const Region &region, X::Pixmap pixmap);
    void buttonPressEvent(Time time, int x, int y, int x_root, int y_root,
			  unsigned int state, unsigned int button);
    void buttonReleaseEvent(Time time, int x, int y, int x_root, int y_root,
			    unsigned int state, unsigned int button);
    void enterEvent(Time time, int x, int y, int x_root, int y_root,
		    unsigned int state/*, char is_hint*/);
    void leaveEvent(Time time, int x, int y, int x_root, int y_root,
		    unsigned int state/*, char is_hint*/);
};

class Button : public GenericButton
{
    std::string _text;
public:
    Button(Connection &conn);
    std::string text() const;
    void setText(const std::string &text);
protected:
    void paintDecoration(int xpos, int ypos, X::Pixmap pixmap);
private:
    CornerRoundness cornerRoundness() const;
    void updateSizeRequirements();
};

} /* namespace Toolkit */

#endif
