/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef TOOLKIT_CONNECTION_H
#define TOOLKIT_CONNECTION_H

#include "x_abstraction.h"

namespace Toolkit {

class WindowHook;
class AbstractMenu;
class Widget;
class WidgetWindowHook;

class ConnectionAtoms
{
public:
    ConnectionAtoms();
    void internAtoms(X::Display display);

    Atom UTF8_STRING;
    Atom TEXT;
    Atom COMPOUND_TEXT;
    Atom CLIPBOARD;
    Atom WM_PROTOCOLS;
    Atom WM_DELETE_WINDOW;

    struct {
	Atom WM_STATE;
	Atom WM_STATE_FULLSCREEN;
    } ewmh;
};

class ConnectionColors
{
public:
    ConnectionColors();
    void allocColors(X::Colormap colormap);

    XColor iconBackground;
    XColor background;
    XColor selected;
    XColor panelBorder;
    XColor panelDark;
    XColor panelLight;
    XColor buttonBorder;
    XColor buttonBorderPixel;
    XColor buttonRaised;
    XColor buttonLowered;
    XColor buttonHighlight;
    XColor buttonDown;
    XColor buttonText;
    XColor scrollbar;
    XColor scrollbarBorder;
    XColor scrollbarRipple0;
    XColor scrollbarRipple1;
    XColor scrollbarArrow;
    XColor scrollbarInsensitiveArrow;
    XColor navigatorSymbol;
    XColor navigatorBackground;
    XColor navigatorBorder;

    XColor inputText;
    XColor inputCursor;
    XColor inputBackground;
    XColor inputSelectionText;
    XColor inputSelectionBackground;
    XColor inputSelectionUnfocussedText;
    XColor inputSelectionUnfocussedBackground;

    XColor selectorText;
    XColor selectorArrow;
    XColor selectorBackground;
    XColor selectorFocussedText;
    XColor selectorFocussedArrow;
    XColor selectorFocussedBackground;

    XColor tabbarSelected;
    XColor tabbarSelectedText;
    XColor tabbarUnselected;
    XColor tabbarUnselectedText;
    XColor tabbarPlasma;

    XColor dockPlasma;
    XColor dockPlasmaText;
    XColor dockTitle;
    XColor dockTitleText;
    XColor dockFocussedTitle;
    XColor dockFocussedTitleText;
    XColor dockToolbar;
    XColor dockToolbarSeparator;
    XColor statusbar;
    XColor statusbarText;
    XColor statusbarProgress;

    XColor menuLabel;
    XColor menuMnemonic;
    XColor menuHotkey;
    XColor menuArrow;
    XColor menuBackground;
    XColor menuHighlightLabel;
    XColor menuHighlightMnemonic;
    XColor menuHighlightHotkey;
    XColor menuHighlightArrow;
    XColor menuHighlight;
    XColor menuDisabledLabel;
    XColor menuDisabledMnemonic;
    XColor menuDisabledHotkey;
    XColor menuDisabledArrow;
    XColor menuDisabled;
    XColor menuBorder;
    XColor menuSeparator;

    XColor toolbar;
    XColor toolbarButton;
    XColor toolbarButtonBorder;
    XColor toolbarButtonArrow;
    XColor toolbarActive;
    XColor toolbarActiveBorder;
    XColor toolbarActiveArrow;
    XColor toolbarHighlight;
    XColor toolbarHighlightBorder;
    XColor toolbarHighlightArrow;
    XColor toolbarHighlightActive;
    XColor toolbarHighlightActiveBorder;
    XColor toolbarHighlightActiveArrow;
    XColor toolbarDisabled;
    XColor toolbarDisabledBorder;
    XColor toolbarDisabledArrow;
    XColor toolbarDisabledActive;
    XColor toolbarDisabledActiveBorder;
    XColor toolbarDisabledActiveArrow;
    XColor toolbarPressed;
    XColor toolbarPressedBorder;
    XColor toolbarPressedArrow;
    XColor toolbarSeparator;
    XColor toolbarOverflow;

    XColor viewBackground;
    XColor viewItemText;
    XColor viewItemBackground0;
    XColor viewItemBackground1;
    XColor viewSelected;
    XColor viewSelectedText;
    XColor viewHighlight;
    XColor viewHighlightText;
    XColor viewLines;
    XColor viewPlupForeground;
    XColor viewPlupBackground;
    XColor viewPlupBorder;

    XColor btnviewBackground;
    XColor btnviewItemBorder;
    XColor btnviewItemText;
    XColor btnviewItemHotkey;
    XColor btnviewItemBackground;
    XColor btnviewCategoryBorder;
    XColor btnviewCategoryText;
    XColor btnviewCategoryHotkey;
    XColor btnviewCategoryArrow;
    XColor btnviewCategoryBackground;
    XColor btnviewHighlightBorder;
    XColor btnviewHighlightText;
    XColor btnviewHighlightHotkey;
    XColor btnviewHighlightBackground;
    XColor btnviewCatHghlghtBorder;
    XColor btnviewCatHghlghtText;
    XColor btnviewCatHghlghtHotkey;
    XColor btnviewCatHghlghtArrow;
    XColor btnviewCatHghlghtBackground;
    XColor btnviewSelectedBorder;
    XColor btnviewSelectedText;
    XColor btnviewSelectedHotkey;
    XColor btnviewSelectedBackground;

    XColor propertiesFixed;
    XColor propertiesFixedText;
    XColor propertiesSelected;
    XColor propertiesSelectedText;
    XColor propertiesSelectedFixed;
    XColor propertiesSelectedFixedText;
};

class Connection
{
    class Impl;
    Impl *const p;
public:
    Connection(char *display_name);
    ~Connection();
    X::Display display() const;
    X::Screen screen() const;

    void addHook(WindowHook *hook);
    void removeHook(WindowHook *hook);
    void update();
    bool abandoned() const;

    Visual *visual() const;
    XVisualInfo const &visualInfo() const;
    ConnectionAtoms const &atoms() const;
    ConnectionColors const &colors() const;
    X::FontSet fontSet() const;
    X::FontSet fontSetBold() const;
    int cachedEscapement(X::FontSet fontSet, const std::wstring &str) const;
    std::wstring ellipsis(X::FontSet fontSet, const std::wstring &str,
			  int width) const;

    AbstractMenu *rootMenu() const;
    void enterMenuMode(Time time, AbstractMenu *menu);
    void leaveMenuMode(Time time);
    void notifyAutomaticGrab(Time time, Widget *widget,
			     WidgetWindowHook *hook);
    void notifyAutomaticUngrab(Time time, WidgetWindowHook *hook);
    void transferAutomaticGrab(Time time, WidgetWindowHook *from_hook,
			       Widget *to_widget);
    void disablePointerGrab();

    X::InputMethod inputMethod() const;
    unsigned long ximStyle() const;

    void handleEvent(const XEvent &event);
};

} /* namespace Toolkit */

#endif
