/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef TOOLKIT_CONTAINER1_H
#define TOOLKIT_CONTAINER1_H

#include "container0.h"

namespace Toolkit {

class Container : public ContainerWidget
{
    int _spacing, _padding;
    bool _expand;
    unsigned long _background;
    std::vector<Control *> ordered_controls;

    virtual void updateSizeRequirements() = 0;
public:
    Container(Connection &conn);
    ~Container();
    int spacing() const;
    void setSpacing(int spacing);
    int padding() const;
    void setPadding(int padding);
    bool expand() const;
    void setExpand(bool expand);
    unsigned long background() const;
    void setBackground(unsigned long background);

    void addWidget(Widget *widget);
    void insertWidget(unsigned int position, Widget *widget);
    void removeWidget(Widget *widget);
    bool widgetEnabled(Widget *widget) const;
    void setWidgetEnabled(bool enabled, Widget *widget);
    void sizeRequirementsChanged(WidgetControl *control);

    void paint(const Region &region, X::Pixmap pixmap);

protected:
    std::vector<Control *> controls() const;
};

class VContainer : public Container
{
    int min_width, min_height;
    int max_width, expanding;
    void updateSizeRequirements();
public:
    VContainer(Connection &conn);
    void updateLayout();
};

class HContainer : public Container
{
    int min_width, min_height;
    int max_height, expanding;
    void updateSizeRequirements();
public:
    HContainer(Connection &conn);
    void updateLayout();
};

} /* namespace Toolkit */

#endif
