/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef TOOLKIT_DOCKS_H
#define TOOLKIT_DOCKS_H

#include "action.h"

namespace Toolkit {
    class Widget;
    class Connection;
    class WidgetControl;
    class Focussable;

    namespace Docks {
	class Dock;
	class Manager;

	/* internal classes */
	class Logic;
	class FocussableControl;
	class FocusHandler;
	class Configuration;

	class DockboxWidget;
	class ColumnWidget;
	class LayoutWidget;
	class MainWidget;
    }
}

class Toolkit::Docks::Dock : public ActionSet
{
    Manager &manager;
public:
    Dock(Manager &manager, const std::string &title, Icon *icon,
	 Widget *widget, const std::string &id,
	 unsigned long textColor, unsigned long backgroundColor);
    ~Dock();
    Action action;
    void actionTriggered(Action *action);

    std::wstring const title;
    Icon *const icon;
    Widget *const widget;
    std::string const id;
    unsigned long const textColor;
    unsigned long const backgroundColor;
};

class Toolkit::Docks::Manager
{
    friend class Dock;
    Logic *const logic;
public:
    Manager(Connection &conn);
    ~Manager();
    Widget *widget() const;
    WidgetControl *mainControl() const;
    Focussable *focusHandler() const;

    Dock *focussedDock() const;
    void setFocussedDock(Dock *dock);

    std::string configuration() const;
    void setConfiguration(const std::string &configuration);
};

#endif
