/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef TOOLKIT_DOCKS_LOGIC_H
#define TOOLKIT_DOCKS_LOGIC_H

#include <map>
#include "mainw.h"
#include "layoutw.h"

class Toolkit::Docks::Logic
{
    MainWidget main_widget;
    LayoutWidget layout_widget;
    std::map<std::string, Dock *> docks_by_id;
    X::Cursor horz_resize_cursor, vert_resize_cursor;
public:
    Logic(Connection &conn);
    ~Logic();
    FocusHandler focusHandler;

    /* Dock */
    void addDock(Dock *dock);
    void removeDock(Dock *dock);

    /* Manager */
    Widget *widget();
    WidgetControl *mainControl();

    Dock *focussedDock() const;
    void setFocussedDock(Dock *dock);

    std::string configuration() const;
    void setConfiguration(const std::string &configuration);

    /* DockboxWidget */
    std::map<Dock *, DockboxWidget *> containingDockbox;
    Dock *lookupDock(const std::string &id);

    /* LayoutWidget */
    X::Cursor horzResizeCursor() const;

    /* ColumnWidget */
    X::Cursor vertResizeCursor() const;
};

#endif
