/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef TOOLKIT_HOTKEY_H
#define TOOLKIT_HOTKEY_H

#include <X11/X.h>
#include <string>
#include <vector>

namespace Toolkit {

class Hotkey
{
    class Step {
	unsigned int state;
	KeySym keysym;

	bool is_glyph;
	wchar_t ch;
	std::string name;
    public:
	Step(unsigned int state, KeySym keysym, KeySym rawKeysym);
	Step(std::string const &repr);

	bool equals(const Step &step) const;
	std::string format() const;

	class Invalid : public std::exception {
	};
    };

    friend bool operator==(const Step &p0, const Step &p1);
    friend bool operator!=(const Step &p0, const Step &p1);

    std::vector<Step> steps;
    bool is_invalid;
    void parse(std::string const &repr);
public:
    Hotkey();
    Hotkey(const Hotkey &continued,
	   unsigned int state, KeySym keysym, KeySym rawKeysym);
    Hotkey(const char *repr);
    Hotkey(std::string const &repr);
    void clear();

    bool isNull() const;
    bool isInvalid() const;
    bool equals(const Hotkey &hotkey) const;
    bool startsWith(const Hotkey &hotkey) const;
    std::string format() const;
};

bool operator==(const Hotkey &h0, const Hotkey &h1);
bool operator!=(const Hotkey &h0, const Hotkey &h1);

} /* namespace Toolkit */

#endif
