/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef TOOLKIT_INPUTS_H
#define TOOLKIT_INPUTS_H

#include "widget.h"

namespace Toolkit {

class Input : public Widget, Focussable
{
    class CursorBlinkTimer;
    class PasteConversion;
    std::wstring _text;
    unsigned int scrolled, cursor_pos;
    unsigned int selection_start, selection_length;
    bool is_clicked;
    Time last_event_time;
    Time multiple_click_time;
    int multiple_click_count;
    unsigned int multiple_click_pos;
    bool cursor_hidden;
    bool has_focus;
    CursorBlinkTimer *const timer;
    void get_word_dimensions(unsigned int pos,
			     unsigned int &start, unsigned int &length) const;
    unsigned int get_pos(int x, int y) const;
    void select_to(unsigned int pos);
    void reset_timer();
    void replace_selection(const std::wstring &str);
    void key_motion(unsigned int pos, unsigned int state);

public:
    Input(Connection &conn);
    ~Input();
    std::string text() const;
    void setText(const std::string &text);

    void paint(const Region &region, X::Pixmap pixmap);

    void motionEvent(Time time, int x, int y, int x_root, int y_root,
		     unsigned int state, char is_hint);
    void buttonPressEvent(Time time, int x, int y, int x_root, int y_root,
			  unsigned int state, unsigned int button);
    void buttonReleaseEvent(Time time, int x, int y, int x_root, int y_root,
			    unsigned int state, unsigned int button);
    void keyPressEvent(Time time, int x, int y, int x_root, int y_root,
		       unsigned int state, unsigned int keycode,
		       const std::string &text, KeySym keysym);
    void keyReleaseEvent(Time time, int x, int y, int x_root, int y_root,
			 unsigned int state, unsigned int keycode);
    void focusInEvent();
    void focusOutEvent();

    void selectionNotify(const std::wstring &str, Time time);

    void setCursorPos(unsigned int cursorPos, bool reset = false);
    void setSelection(unsigned int selectionStart,
		      unsigned int selectionLength,
		      unsigned int cursorPos, bool reset = false);
    void selectAll();
    void selectNone();
    void blink();
};

} /* namespace Toolkit */

#endif
