/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef TOOLKIT_MODEL_VIEW_H
#define TOOLKIT_MODEL_VIEW_H

#include <string>
#include <vector>

#define NPOS static_cast<unsigned int>(-1)

class ItemInterior;
class Icon;
class ItemEditor;
class Shadow;

class Item
{
    ItemInterior *_interior;
    std::wstring _caption, _hotkey;
    Icon *_icon;
    bool is_bold, is_italic;
    ItemEditor *_editor;
    std::vector<Shadow *> shadows;

    friend class Shadow;
    void add_shadow(Shadow *shadow);
    void remove_shadow(Shadow *shadow);
public:
    Item();
    virtual ~Item();
    ItemInterior *interior() const;
    std::wstring caption() const;
    std::wstring hotkey() const;
    Icon *icon() const;
    bool isBold() const;
    bool isItalic() const;
    ItemEditor *editor() const;
protected:
    void setInterior(ItemInterior *interior);
    void setCaption(const std::string &caption);
    void setHotkey(const std::string &hotkey);
    void setIcon(Icon *icon);
    void setBold(bool bold);
    void setItalic(bool italic);
    void setEditor(ItemEditor *editor);
};

class ItemInterior
{
    virtual Item *item(unsigned int index) const = 0;
    virtual unsigned int indexOf(Item *item) const = 0;

    mutable std::vector<Item *> item_cache;
    std::vector<Shadow *> shadows;

    friend class Shadow;
    void add_shadow(Shadow *shadow);
    void remove_shadow(Shadow *shadow);
public:
    virtual ~ItemInterior();
    Item *cachedItem(unsigned int index) const;
    unsigned int cachedIndexOf(Item *item) const;
    unsigned int count() const;
protected:
    void replace(unsigned int index, unsigned int old_count,
				     unsigned int new_count);
};

class Shadow
{
    virtual void updateItem() = 0;
    virtual void updateItems(unsigned int index, unsigned int old_count,
						 unsigned int new_count) = 0;
    virtual Shadow *buildShadow(Item *item) const = 0;

    Item *const _item;
    Shadow *const _parent;
    ItemInterior *_interior;
    mutable std::vector<Shadow *> shadow_cache;
    void set_interior(ItemInterior *interior);

    friend class Item;
    void update_interior();

    friend class ItemInterior;
    void update_items(unsigned int index, unsigned int old_count,
					  unsigned int new_count);
public:
    Shadow(ItemInterior *interior);
    Shadow(Item *item, Shadow *parent);
    virtual ~Shadow();

    Item *item() const;
    Shadow *parent() const;

    ItemInterior *interior() const;
    unsigned int indexOf(Shadow *shadow) const;
    Shadow *shadow(unsigned int index) const;
    unsigned int count() const;
    Shadow *lookup(const std::vector<Item *> &items);
};

#endif
