/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef TOOLKIT_NAVIGATOR_H
#define TOOLKIT_NAVIGATOR_H

#include "widget.h"

namespace Toolkit {

class NavigatorData
{
    int view_x, view_y, view_width, view_height;
    int content_width, content_height;
    X::Pixmap _pixmap;
    void apply_constraints();
public:
    NavigatorData();
    NavigatorData(int view_x, int view_y, int view_width, int view_height,
		  int content_width, int content_height, X::Pixmap pixmap);

    int viewX() const;
    int viewY() const;
    int viewWidth() const;
    int viewHeight() const;
    int contentWidth() const;
    int contentHeight() const;
    X::Pixmap pixmap() const;

    void setPosition(int x, int y);
};

class Navigator : public Widget
{
    class Operation;
    Operation *op;

    virtual NavigatorData getNavigatorData() const = 0;
    virtual void navigateTo(int x, int y) = 0;
public:
    Navigator(Connection &conn);
    ~Navigator();
    void paint(const Region &region, X::Pixmap pixmap);

    void motionEvent(Time time, int x, int y, int x_root, int y_root,
		     unsigned int state, char is_hint);
    void buttonPressEvent(Time time, int x, int y, int x_root, int y_root,
			  unsigned int state, unsigned int button);
    void buttonReleaseEvent(Time time, int x, int y, int x_root, int y_root,
			    unsigned int state, unsigned int button);
};

} /* namespace Toolkit */

#endif
