/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef TOOLKIT_SCROLLING_H
#define TOOLKIT_SCROLLING_H

#include "buttons.h"
#include "container0.h"

namespace Toolkit {

class Scrollbar;

class ScrollbarHandle : public ButtonicWidget
{
    Scrollbar &scrollbar;
    int grab_x, grab_y;
    bool is_grabbed;
public:
    ScrollbarHandle(Scrollbar &scrollbar);
    bool isGrabbed() const;

    void buttonPressEvent(Time time, int x, int y, int x_root, int y_root,
			  unsigned int state, unsigned int button);
    void buttonReleaseEvent(Time time, int x, int y, int x_root, int y_root,
			    unsigned int state, unsigned int button);
    void motionEvent(Time time, int x, int y, int x_root, int y_root,
		     unsigned int state, char is_hint);
protected:
    void paintDecoration(int xpos, int ypos, X::Pixmap pixmap);
private:
    CornerRoundness cornerRoundness() const;
};

class ScrollbarButton : public GenericButton
{
public:
    ScrollbarButton(Scrollbar &scrollbar);
    enum Type { Left, Right, Up, Down };
    Type type() const;
    void setType(Type type);
    bool insensitive() const;
    void setInsensitive(bool insensitive);

    void buttonPressEvent(Time time, int x, int y, int x_root, int y_root,
			  unsigned int state, unsigned int button);
protected:
    void paintDecoration(int xpos, int ypos, X::Pixmap pixmap);
private:
    Scrollbar &scrollbar;
    Type _type;
    CornerRoundness cornerRoundness() const;
};

class Scrollbar : public ContainerWidget
{
public:
    enum Type {
	Top, Left, Bottom, Right
    };

    class Scroller : public WidgetControlScroller
    {
	Scrollbar &scrollbar;
    public:
	Scroller(Scrollbar &scrollbar, WidgetControl &control);
	void updateRange();
	void updatePageSize();
	void updatePosition();
    };

    Scrollbar(Type type, WidgetControl &control, Connection &conn);
    Scroller scroller;
    Type type() const;

    int range() const;
    int pageSize() const;
    int position() const;
    void setPosition(int position);
    int lineSize() const;

    void updateLayout();
    void paint(const Region &region, X::Pixmap pixmap);

    void buttonPressEvent(Time time, int x, int y, int x_root, int y_root,
			  unsigned int state, unsigned int button);

    void moveHandle(int delta_x, int delta_y);
    void releaseHandle();

private:
    Type const _type;
    Control dec_wc, inc_wc, handle_wc;
    ScrollbarButton dec, inc;
    ScrollbarHandle handle;

    unsigned int handle_size() const;
    WidgetScroller *widget_scroller() const;
    void paint1(int t0, int t1, const Region &region, X::Pixmap pixmap);
    void parameters_changed();
    void update_handle_dimensions();
};

class ScrollPane : public ContainerWidget
{
public:
    ScrollPane(Connection &conn);
    WidgetControl *mainControl();
    WidgetControl *navigatorControl();
    unsigned long background() const;
    void setBackground(unsigned long background);

    void updateLayout();
    void sizeRequirementsChanged(WidgetControl *control);
    void paint(const Region &region, X::Pixmap pixmap);

private:
    class MainControl : public ContainerWidget::Control
    {
	ScrollPane &pane;
	void widgetChanged();
    public:
	MainControl(ScrollPane &pane);
    };
    MainControl main_wc;
    Control horz_wc, vert_wc, nav_wc;
    Scrollbar horz_sb, vert_sb;
    unsigned long _background;
};

} /* namespace Toolkit */

#endif
