/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef TOOLKIT_SELECTION_H
#define TOOLKIT_SELECTION_H

#include "x_abstraction.h"

namespace Toolkit {

class SelectionWindowHook;
class Connection;

class SelectionHook
{
public:
    SelectionHook(Atom selection, Time time);
    virtual ~SelectionHook();

    Atom const selection;
    Time const time;

    virtual bool request(X::Window requestor, Atom target,
			 Atom property, Time time);
    virtual void clear(Time time);
};

class ConversionHook
{
public:
    ConversionHook(Atom selection, Atom target, Time time);
    virtual ~ConversionHook();

    Atom const selection;
    Atom const target;
    Time const time;

    virtual void notify(Atom property, Time time);
    virtual void cancel(Time time);
};

class ComplexConversion
{
    class Hook;
    SelectionWindowHook *const _window;
    Atom const _selection;
    std::vector<Atom> const targets;
    unsigned int target_index;

public:
    ComplexConversion(SelectionWindowHook *window, Atom selection,
		      const std::vector<Atom> &targets);
    virtual ~ComplexConversion();
    ConversionHook *createHook(Time time);

    virtual void notify(Atom property, Time time);
    virtual void cancel(Time time);

    SelectionWindowHook *window() const;
    Atom selection() const;

protected:
    Atom target() const;
};

class StringSelectionHook : public SelectionHook
{
    Connection &conn;
    std::string const str;
public:
    StringSelectionHook(Connection &conn, Atom selection, Time time,
			const std::string &str);

    bool request(X::Window requestor, Atom target,
		 Atom property, Time time);
};

class StringConversion : public ComplexConversion
{
    Connection &conn;
    virtual void notifySuccess(const std::string &str, Time time);
    virtual void notifyFailure(Time time);
public:
    StringConversion(Connection &conn, SelectionWindowHook *window,
		     Atom selection);
    void notify(Atom property, Time time);
};

class WideStringSelectionHook : public SelectionHook
{
    Connection &conn;
    std::wstring const str;
public:
    WideStringSelectionHook(Connection &conn, Atom selection, Time time,
			    const std::wstring &str);

    bool request(X::Window requestor, Atom target,
		 Atom property, Time time);
};

class WideStringConversion : public ComplexConversion
{
    Connection &conn;
    virtual void notifySuccess(const std::wstring &str, Time time);
    virtual void notifyFailure(Time time);
public:
    WideStringConversion(Connection &conn, SelectionWindowHook *window,
			 Atom selection);
    void notify(Atom property, Time time);
};

} /* namespace Toolkit */

#endif
