/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#include <algorithm>
#include "stdmodel.h"


StandardItem::StandardItem(const std::string &caption, Icon *icon)
{
    setInterior(&interior);
    setCaption(caption);
    setIcon(icon);
}

void StandardItem::setCaption(const std::string &caption)
{
    Item::setCaption(caption);
}

void StandardItem::setHotkey(const std::string &hotkey)
{
    Item::setHotkey(hotkey);
}

void StandardItem::setIcon(Icon *icon)
{
    Item::setIcon(icon);
}

void StandardItem::setBold(bool bold)
{
    Item::setBold(bold);
}

void StandardItem::setItalic(bool italic)
{
    Item::setItalic(italic);
}

void StandardItem::setEditor(ItemEditor *editor)
{
    Item::setEditor(editor);
}

/****************************************************************************/

StandardInterior::StandardInterior(bool deleteItems)
    : delete_items(deleteItems)
{
}

StandardInterior::StandardInterior(const std::vector<Item *> &items,
				   bool deleteItems)
    : _items(items), delete_items(deleteItems)
{
    ItemInterior::replace(0, 0, items.size());
}

StandardInterior::StandardInterior(const std::deque<Item *> &items,
				   bool deleteItems)
    : delete_items(deleteItems)
{
    for (std::deque<Item *>::const_iterator i = items.begin();
	 i != items.end(); i++)
	_items.push_back(*i);

    ItemInterior::replace(0, 0, items.size());
}

StandardInterior::~StandardInterior()
{
    if (delete_items)
	for (std::vector<Item *>::const_iterator i = _items.begin();
	     i != _items.end(); i++)
	    delete *i;
}

Item *StandardInterior::item(unsigned int index) const
{
    return _items[index];
}

unsigned int StandardInterior::indexOf(Item *item) const
{
    std::vector<Item *>::const_iterator i =
	std::find(_items.begin(), _items.end(), item);
    return i == _items.end() ? -1 : i - _items.begin();
}

std::vector<Item *> StandardInterior::items() const
{
    return _items;
}

void StandardInterior::replace(unsigned int index, unsigned int count,
			       const std::vector<Item *> &items)
{
    _items.insert(_items.erase(_items.begin() + index,
			       _items.begin() + index + count),
		  items.begin(), items.end());

    ItemInterior::replace(index, count, items.size());
}

/****************************************************************************/

MutableStandardInterior::MutableStandardInterior(bool deleteItems)
    : StandardInterior(deleteItems)
{
}

std::vector<Item *> MutableStandardInterior::items() const
{
    return StandardInterior::items();
}

void MutableStandardInterior::setItems(const std::vector<Item *> &items)
{
    StandardInterior::replace(0, count(), items);
}

void MutableStandardInterior::addItem(Item *item)
{
    std::vector<Item *> items;
    items.push_back(item);
    StandardInterior::replace(count(), 0, items);
}

void MutableStandardInterior::replace(unsigned int index, unsigned int count,
				      const std::vector<Item *> &items)
{
    StandardInterior::replace(index, count, items);
}
