/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#include <stdlib.h>
#include <err.h>
#include "encoding.h"


bool encode(const std::wstring &in, std::string &out)
{
    size_t len = in.length() * 16 + 1;
    char buf[len];
    const wchar_t *src = in.c_str();

    size_t used = wcsrtombs(buf, &src, len, NULL);

    if (used == (size_t)-1)
	return false;

    out = std::string(buf, used);
    return true;
}

bool decode(const std::string &in, std::wstring &out)
{
    size_t len = (in.length() + 1) * sizeof(wchar_t);
    wchar_t buf[len];
    const char *src = in.c_str();

    size_t used = mbsrtowcs(buf, &src, len, NULL);

    if (used == (size_t)-1)
	return false;

    out = std::wstring(buf, used);
    return true;
}

std::string warn_encode(const std::wstring &str)
{
    std::string res;
    if (!encode(str, res))
	warn("wcsrtombs");
    return res;
}

std::wstring warn_decode(const std::string &str)
{
    std::wstring res;
    if (!decode(str, res))
	warn("mbsrtowcs");
    return res;
}

std::string assert_encode(const std::wstring &str)
{
    std::string res;
    if (!encode(str, res))
	err(EXIT_FAILURE, "wcsrtombs");
    return res;
}

std::wstring assert_decode(const std::string &str)
{
    std::wstring res;
    if (!decode(str, res))
	err(EXIT_FAILURE, "mbsrtowcs");
    return res;
}
