/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#include "region.h"

using namespace Toolkit;


Region::Region() : x(0), y(0), width(0), height(0)
{
}

Region::Region(int x, int y, int width, int height)
    : x(x), y(y), width(width), height(height)
{
}

bool Region::contains(int px, int py) const
{
    return px >= x && px < x + width &&
	   py >= y && py < y + height;
}

bool Region::contains(const Region &region) const
{
    return region.x >= x && x + width >= region.x + region.width &&
	   region.y >= y && y + height >= region.y + region.height;
}

bool Region::overlaps(const Region &region) const
{
    /* Possible configurations are []<> [<]> [<>] <[]> <[>] <>[] */

    return (region.x < x + width) && (x < region.x + region.width) &&
	   (region.y < y + height) && (y < region.y + region.height);
}

Region Region::intersection(const Region &region) const
{
    if (!overlaps(region))
	return Region();

    int left = x > region.x ? x : region.x;
    int top = y > region.y ? y : region.y;

    return Toolkit::Region(
	left, top, (x + width < region.x + region.width ?
		    x + width : region.x + region.width) - left,
		   (y + height < region.y + region.height ?
		    y + height : region.y + region.height) - top);
}

Region Region::hull(const Region &region) const
{
    if (region.width == 0 || region.height == 0)
	return *this;
    if (width == 0 || height == 0)
	return region;

    int left = x > region.x ? region.x : x;
    int top = y > region.y ? region.y : y;

    return Toolkit::Region(
	left, top, (x + width < region.x + region.width ?
		    region.x + region.width : x + width) - left,
		   (y + height < region.y + region.height ?
		    region.y + region.height : y + height) - top);
}

bool Toolkit::operator==(const Region &r0, const Region &r1)
{
    return r0.x == r1.x && r0.width == r1.width &&
	   r0.y == r1.y && r0.height == r1.height;
}

bool Toolkit::operator!=(const Region &r0, const Region &r1)
{
    return r0.x != r1.x || r0.width != r1.width ||
	   r0.y != r1.y || r0.height != r1.height;
}
