/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#include "size_reqs.h"

using namespace Toolkit;


SizeRequirements::SizeRequirements() : minWidth(0), minHeight(0),
				       maxWidth(-1), maxHeight(-1)
{
}

void SizeRequirements::addX(unsigned int x)
{
    minWidth += x;
    if (maxWidth != -1)
	maxWidth += x;
}

void SizeRequirements::addX(const SizeRequirements &reqs)
{
    minWidth += reqs.minWidth;

    if (reqs.minHeight > minHeight)
	minHeight = reqs.minHeight;

    if (maxWidth != -1) {
	if (reqs.maxWidth == -1)
	    maxWidth = -1;
	else
	    maxWidth += reqs.maxWidth;
    }

    if (reqs.maxHeight != -1 && (
	    maxHeight == -1 || reqs.maxHeight < maxHeight))
	maxHeight = reqs.maxHeight;

    if (maxWidth != -1 && minWidth > maxWidth)
	maxWidth = minWidth;
    if (maxHeight != -1 && minHeight > maxHeight)
	maxHeight = minHeight;
}

void SizeRequirements::addY(unsigned int y)
{
    minHeight += y;
    if (maxHeight != -1)
	maxHeight += y;
}

void SizeRequirements::addY(const SizeRequirements &reqs)
{
    if (reqs.minWidth > minWidth)
	minWidth = reqs.minWidth;

    minHeight += reqs.minHeight;

    if (reqs.maxWidth != -1 && (
	    maxWidth == -1 || reqs.maxWidth < maxWidth))
	maxWidth = reqs.maxWidth;

    if (maxHeight != -1) {
	if (reqs.maxHeight == -1)
	    maxHeight = -1;
	else
	    maxHeight += reqs.maxHeight;
    }

    if (maxWidth != -1 && minWidth > maxWidth)
	maxWidth = minWidth;
    if (maxHeight != -1 && minHeight > maxHeight)
	maxHeight = minHeight;
}

bool Toolkit::operator==(const SizeRequirements &r0,
			 const SizeRequirements &r1)
{
    return r0.minWidth == r1.minWidth && r0.minHeight == r1.minHeight &&
	   r0.maxWidth == r1.maxWidth && r0.maxHeight == r1.maxHeight;
}

bool Toolkit::operator!=(const SizeRequirements &r0,
			 const SizeRequirements &r1)
{
    return r0.minWidth != r1.minWidth || r0.minHeight != r1.minHeight ||
	   r0.maxWidth != r1.maxWidth || r0.maxHeight != r1.maxHeight;
}
