/* Copyright (C) 2013 Roland Lutz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#include <stdlib.h>
#include "x_abstraction.h"

namespace X {

Display Display::open(const char *display_name)
{
    return Display(XOpenDisplay(display_name));
}

Display::Display() : display(NULL)
{
}

Display::Display(::Display *display) : display(display)
{
}

Display::operator ::Display*() const
{
    return display;
}

bool Display::isNull() const
{
    return !display;
}

int Display::close()
{
    return XCloseDisplay(display);
}

int Display::connectionNumber()
{
    return ConnectionNumber(display);
}

Screen Display::defaultScreen()
{
    return Screen(display, XDefaultScreen(display));
}

Window Display::defaultRootWindow()
{
    return Window(display, XDefaultRootWindow(display));
}

int Display::nextEvent(XEvent *event_return)
{
    return XNextEvent(display, event_return);
}

int Display::peekEvent(XEvent *event_return)
{
    return XPeekEvent(display, event_return);
}

int Display::maskEvent(long event_mask, XEvent *event_return)
{
    return XMaskEvent(display, event_mask, event_return);
}

bool Display::checkMaskEvent(long event_mask, XEvent *event_return)
{
    return XCheckMaskEvent(display, event_mask, event_return);
}

bool Display::checkTypedEvent(int event_type, XEvent *event_return)
{
    return XCheckTypedEvent(display, event_type, event_return);
}

int Display::flush()
{
    return XFlush(display);
}

int Display::sync(bool discard)
{
    return XSync(display, discard);
}

int Display::eventsQueued(int mode)
{
    return XEventsQueued(display, mode);
}

int Display::pending()
{
    return XPending(display);
}

Cursor Display::createFontCursor(unsigned int shape)
{
    return Cursor(display, XCreateFontCursor(display, shape));
}

Cursor Display::createPixmapCursor(
    ::Pixmap source, ::Pixmap mask,
    XColor *foreground_color, XColor *background_color,
    unsigned int x, unsigned int y)
{
    return Cursor(display, XCreatePixmapCursor(
		      display, source, mask,
		      foreground_color, background_color, x, y));
}

Cursor Display::createGlyphCursor(
    ::Font source_font, ::Font mask_font,
    unsigned int source_char, unsigned int mask_char,
    XColor *foreground_color, XColor *background_color)
{
    return Cursor(display, XCreateGlyphCursor(
		      display, source_font, mask_font, source_char, mask_char,
		      foreground_color, background_color));
}

Font Display::loadFont(const char *name)
{
    return Font(display, XLoadFont(display, name));
}

XFontStruct *Display::queryFont(XID font_ID)
{
    return XQueryFont(display, font_ID);
}

XFontStruct *Display::loadQueryFont(const char *name)
{
    return XLoadQueryFont(display, name);
}

int Display::freeFont(XFontStruct *font_struct)
{
    return XFreeFont(display, font_struct);
}

int Display::ungrabPointer(Time time)
{
    return XUngrabPointer(display, time);
}

int Display::changeActivePointerGrab(unsigned int event_mask,
				     ::Cursor cursor, Time time)
{
    return XChangeActivePointerGrab(display, event_mask, cursor, time);
}

FontSet Display::createFontSet(const char *base_font_name_list,
			       char ***missing_charset_list_return,
			       int *missing_charset_count_return,
			       char **def_string_return)
{
    return FontSet(display, XCreateFontSet(display, base_font_name_list,
					   missing_charset_list_return,
					   missing_charset_count_return,
					   def_string_return));
}

InputMethod Display::openIM(struct _XrmHashBucketRec *db,
			    char *res_name, char *res_class)
{
    return InputMethod(XOpenIM(display, db, res_name, res_class));
}

Atom Display::internAtom(const char *atom_name, Bool only_if_exists)
{
    return XInternAtom(display, atom_name, only_if_exists);
}

Status Display::internAtoms(char **names, int count, Bool only_if_exists,
			    Atom *atoms_return)
{
    return XInternAtoms(display, names, count, only_if_exists, atoms_return);
}

char *Display::getAtomName(Atom atom)
{
    return XGetAtomName(display, atom);
}

Status Display::getAtomNames(Atom *atoms, int count, char **names_return)
{
    return XGetAtomNames(display, atoms, count, names_return);
}

int Display::mbTextListToTextProperty(char **list, int count,
    XICCEncodingStyle style, XTextProperty *text_prop_return)
{
    return XmbTextListToTextProperty(
	display, list, count, style, text_prop_return);
}

int Display::wcTextListToTextProperty(wchar_t **list, int count,
    XICCEncodingStyle style, XTextProperty *text_prop_return)
{
    return XwcTextListToTextProperty(
	display, list, count, style, text_prop_return);
}

int Display::utf8TextListToTextProperty(char **list, int count,
    XICCEncodingStyle style, XTextProperty *text_prop_return)
{
    return Xutf8TextListToTextProperty(
	display, list, count, style, text_prop_return);
}

int Display::mbTextPropertyToTextList(XTextProperty *text_prop,
    char ***list_return, int *count_return)
{
    return XmbTextPropertyToTextList(
	display, text_prop, list_return, count_return);
}

int Display::wcTextPropertyToTextList(XTextProperty *text_prop,
    wchar_t ***list_return, int *count_return)
{
    return XwcTextPropertyToTextList(
	display, text_prop, list_return, count_return);
}

int Display::utf8TextPropertyToTextList(XTextProperty *text_prop,
    char ***list_return, int *count_return)
{
    return Xutf8TextPropertyToTextList(
	display, text_prop, list_return, count_return);
}

int Display::setSelectionOwner(Atom selection, ::Window owner, Time time)
{
    return XSetSelectionOwner(display, selection, owner, time);
}

Window Display::getSelectionOwner(Atom selection)
{
    return Window(display, XGetSelectionOwner(display, selection));
}

int Display::convertSelection(Atom selection, Atom target,
			      Atom property, ::Window requestor, Time time)
{
    return XConvertSelection(display, selection, target,
			     property, requestor, time);
}

std::vector<XVisualInfo> Display::getVisualInfo(long vinfo_mask,
						XVisualInfo *vinfo_template)
{
    int nitems_return = 0;
    XVisualInfo *visual_info = XGetVisualInfo(display, vinfo_mask,
					      vinfo_template, &nitems_return);
    std::vector<XVisualInfo> v;

    for (int i = 0; i < nitems_return; i++)
	v.push_back(visual_info[i]);
    XFree(visual_info);

    return v;
}

std::vector<XPixmapFormatValues> Display::listPixmapFormats()
{
    int count_return = 0;
    XPixmapFormatValues *pixmap_format_values =
	XListPixmapFormats(display, &count_return);
    std::vector<XPixmapFormatValues> v;

    for (int i = 0; i < count_return; i++)
	v.push_back(pixmap_format_values[i]);
    XFree(pixmap_format_values);

    return v;
}

int Display::imageByteOrder()
{
    return ImageByteOrder(display);
}

int Display::bitmapBitOrder()
{
    return BitmapBitOrder(display);
}

int Display::bitmapPad()
{
    return BitmapPad(display);
}

int Display::bitmapUnit()
{
    return BitmapUnit(display);
}

/****************************************************************************/

Screen::Screen() : display(NULL), screen_number(0)
{
}

Screen::Screen(::Display *display, int screen_number)
    : display(display), screen_number(screen_number)
{
}

Screen::operator int() const
{
    return screen_number;
}

unsigned long Screen::blackPixel()
{
    return XBlackPixel(display, screen_number);
}

unsigned long Screen::whitePixel()
{
    return XWhitePixel(display, screen_number);
}

Window Screen::rootWindow()
{
    return Window(display, RootWindow(display, screen_number));
}

int Screen::defaultDepth()
{
    return DefaultDepth(display, screen_number);
}

Visual *Screen::defaultVisual()
{
    return DefaultVisual(display, screen_number);
}

Colormap Screen::defaultColormap()
{
    return Colormap(display, DefaultColormap(display, screen_number));
}

Status Screen::matchVisualInfo(int depth, int c_class,
			       XVisualInfo *vinfo_return)
{
    return XMatchVisualInfo(display, screen_number, depth, c_class,
			    vinfo_return);
}

int Screen::displayHeight()
{
    return DisplayHeight(display, screen_number);
}

int Screen::displayHeightMM()
{
    return DisplayHeightMM(display, screen_number);
}

int Screen::displayWidth()
{
    return DisplayWidth(display, screen_number);
}

int Screen::displayWidthMM()
{
    return DisplayWidthMM(display, screen_number);
}

/****************************************************************************/

GC::GC() : display(NULL), gc(0)
{
}

GC::GC(::Display *display, ::GC gc) : display(display), gc(gc)
{
}

GC::operator ::GC() const
{
    return gc;
}

bool GC::isNull() const
{
    return !display;
}

int GC::setState(unsigned long foreground, unsigned int background,
		 int function, unsigned long plane_mask)
{
    return XSetState(display, gc,
		     foreground, background, function, plane_mask);
}

int GC::setFunction(int function)
{
    return XSetFunction(display, gc, function);
}

int GC::setPlaneMask(unsigned long plane_mask)
{
    return XSetPlaneMask(display, gc, plane_mask);
}

int GC::setForeground(unsigned long foreground)
{
    return XSetForeground(display, gc, foreground);
}

int GC::setBackground(unsigned long background)
{
    return XSetBackground(display, gc, background);
}

int GC::copy(GC src, GC dest, unsigned long valuemask)
{
    return XCopyGC(src.display, src, valuemask, dest);
}

int GC::change(unsigned long valuemask, XGCValues *values)
{
    return XChangeGC(display, gc, valuemask, values);
}

Status GC::getValues(unsigned long valuemask, XGCValues *values_return)
{
    return XGetGCValues(display, gc, valuemask, values_return);
}

int GC::free()
{
    return XFreeGC(display, gc);
}

GContext GC::gContext()
{
    return XGContextFromGC(gc);
}

int GC::setFont(::Font font)
{
    return XSetFont(display, gc, font);
}

int GC::setClipOrigin(int clip_x_origin, int clip_y_origin)
{
    return XSetClipOrigin(display, gc, clip_x_origin, clip_y_origin);
}

int GC::setClipMask(::Pixmap pixmap)
{
    return XSetClipMask(display, gc, pixmap);
}

int GC::setClipRectangles(int clip_x_origin, int clip_y_origin,
			  XRectangle rectangles[], int n, int ordering)
{
    return XSetClipRectangles(display, gc, clip_x_origin, clip_y_origin,
			      rectangles, n, ordering);
}

/****************************************************************************/

FontSet::FontSet() : display(NULL), font_set(NULL), _extents(NULL)
{
}

FontSet::FontSet(::Display *display, XFontSet font_set)
    : display(display), font_set(font_set),
      _extents(XExtentsOfFontSet(font_set))
{
}

FontSet::operator XFontSet() const
{
    return font_set;
}

bool FontSet::isNull() const
{
    return !display;
}

int FontSet::fonts(XFontStruct ***font_struct_list_return,
		   char ***font_name_list_return)
{
    return XFontsOfFontSet(font_set, font_struct_list_return,
				     font_name_list_return);
}

char *FontSet::baseFontNameList()
{
    return XBaseFontNameListOfFontSet(font_set);
}

char *FontSet::locale()
{
    return XLocaleOfFontSet(font_set);
}

bool FontSet::contextDependentDrawing()
{
    return XContextDependentDrawing(font_set) != False;
}

bool FontSet::contextualDrawing()
{
    return XContextualDrawing(font_set) != False;
}

bool FontSet::directionalDependentDrawing()
{
    return XDirectionalDependentDrawing(font_set) != False;
}

XFontSetExtents *FontSet::extents() const
{
    return _extents;
}

int FontSet::mbTextEscapement(const std::string &string)
{
    return XmbTextEscapement(font_set, string.c_str(), string.length());
}

int FontSet::wcTextEscapement(const std::wstring &string)
{
    return XwcTextEscapement(font_set, string.c_str(), string.length());
}

int FontSet::utf8TextEscapement(const std::string &string)
{
    return Xutf8TextEscapement(font_set, string.c_str(), string.length());
}

int FontSet::mbTextExtents(const std::string &string,
			   XRectangle *overall_ink_return,
			   XRectangle *overall_logical_return)
{
    return XmbTextExtents(
	font_set, string.c_str(), string.length(),
	overall_ink_return, overall_logical_return);
}

int FontSet::wcTextExtents(const std::wstring &string,
			   XRectangle *overall_ink_return,
			   XRectangle *overall_logical_return)
{
    return XwcTextExtents(
	font_set, string.c_str(), string.length(),
	overall_ink_return, overall_logical_return);
}

int FontSet::utf8TextExtents(const std::string &string,
			     XRectangle *overall_ink_return,
			     XRectangle *overall_logical_return)
{
    return Xutf8TextExtents(
	font_set, string.c_str(), string.length(),
	overall_ink_return, overall_logical_return);
}

Status FontSet::mbTextPerCharExtents(const std::string &string,
				     XRectangle *ink_array_return,
				     XRectangle *logical_array_return,
				     int array_size, int *num_chars_return,
				     XRectangle *overall_ink_return,
				     XRectangle *overall_logical_return)
{
    return XmbTextPerCharExtents(
	font_set, string.c_str(), string.length(), ink_array_return,
	logical_array_return, array_size, num_chars_return,
	overall_ink_return, overall_logical_return);
}

Status FontSet::wcTextPerCharExtents(const std::wstring &string,
				     XRectangle *ink_array_return,
				     XRectangle *logical_array_return,
				     int array_size, int *num_chars_return,
				     XRectangle *overall_ink_return,
				     XRectangle *overall_logical_return)
{
    return XwcTextPerCharExtents(
	font_set, string.c_str(), string.length(), ink_array_return,
	logical_array_return, array_size, num_chars_return,
	overall_ink_return, overall_logical_return);
}

Status FontSet::utf8TextPerCharExtents(const std::string &string,
				       XRectangle *ink_array_return,
				       XRectangle *logical_array_return,
				       int array_size, int *num_chars_return,
				       XRectangle *overall_ink_return,
				       XRectangle *overall_logical_return)
{
    return Xutf8TextPerCharExtents(
	font_set, string.c_str(), string.length(), ink_array_return,
	logical_array_return, array_size, num_chars_return,
	overall_ink_return, overall_logical_return);
}

void FontSet::free()
{
    XFreeFontSet(display, font_set);
}

int FontSet::ascent() const
{
    return -extents()->max_logical_extent.y;
}

int FontSet::descent() const
{
    return extents()->max_logical_extent.y +
	   extents()->max_logical_extent.height;
}

/****************************************************************************/

InputMethod::InputMethod() : im(NULL)
{
}

InputMethod::InputMethod(XIM im) : im(im)
{
}

InputMethod::operator XIM() const
{
    return im;
}

bool InputMethod::isNull() const
{
    return !im;
}

Status InputMethod::close()
{
    return XCloseIM(im);
}

::Display *InputMethod::display()
{
    return XDisplayOfIM(im);
}

char *InputMethod::locale()
{
    return XLocaleOfIM(im);
}

/****************************************************************************/

InputContext::InputContext() : ic(NULL)
{
}

InputContext::InputContext(XIC ic) : ic(ic)
{
}

InputContext::operator XIC() const
{
    return ic;
}

bool InputContext::isNull() const
{
    return !ic;
}

void InputContext::destroy()
{
    XDestroyIC(ic);
}

InputMethod InputContext::IM()
{
    return InputMethod(XIMOfIC(ic));
}

void InputContext::setFocus()
{
    XSetICFocus(ic);
}

void InputContext::unsetFocus()
{
    XUnsetICFocus(ic);
}

std::string InputContext::mbReset()
{
    char *tmp0 = XmbResetIC(ic);
    if (!tmp0)
	return std::string();
    std::string tmp1(tmp0);
    XFree(tmp0);
    return tmp1;
}

std::wstring InputContext::wcReset()
{
    wchar_t *tmp0 = XwcResetIC(ic);
    if (!tmp0)
	return std::wstring();
    std::wstring tmp1(tmp0);
    XFree(tmp0);
    return tmp1;
}

std::string InputContext::utf8Reset()
{
    char *tmp0 = Xutf8ResetIC(ic);
    if (!tmp0)
	return std::string();
    std::string tmp1(tmp0);
    XFree(tmp0);
    return tmp1;
}

std::string InputContext::mbLookupString(const XKeyPressedEvent *event,
					 KeySym *keysym_return,
					 Status *status_return)
{
    static char *buffer = NULL;
    static int bytes_buffer;
    Status status;
    int len;

    if (!buffer)
	buffer = (char *)malloc(bytes_buffer = 16);

    if (!status_return)
	status_return = &status;

    do {
	len = XmbLookupString(ic, const_cast<XKeyPressedEvent *>(event),
			      buffer, bytes_buffer,
			      keysym_return, status_return);
	if (*status_return == XBufferOverflow)
	    buffer = (char *)realloc(buffer, bytes_buffer = len);
    } while (*status_return == XBufferOverflow);

    return std::string(buffer, len);
}

std::wstring InputContext::wcLookupString(const XKeyPressedEvent *event,
					  KeySym *keysym_return,
					  Status *status_return)
{
    static wchar_t *buffer = NULL;
    static int wchars_buffer;
    Status status;
    int len;

    if (!buffer)
	buffer = (wchar_t *)malloc((wchars_buffer = 16) * sizeof(wchar_t));

    if (!status_return)
	status_return = &status;

    do {
	len = XwcLookupString(ic, const_cast<XKeyPressedEvent *>(event),
			      buffer, wchars_buffer,
			      keysym_return, status_return);
	if (*status_return == XBufferOverflow)
	    buffer = (wchar_t *)realloc(
		buffer, (wchars_buffer = len) * sizeof(wchar_t));
    } while (*status_return == XBufferOverflow);

    return std::wstring(buffer, len);
}

std::string InputContext::utf8LookupString(const XKeyPressedEvent *event,
					   KeySym *keysym_return,
					   Status *status_return)
{
    static char *buffer = NULL;
    static int bytes_buffer;
    Status status;
    int len;

    if (!buffer)
	buffer = (char *)malloc(bytes_buffer = 16);

    if (!status_return)
	status_return = &status;

    do {
	len = Xutf8LookupString(ic, const_cast<XKeyPressedEvent *>(event),
				buffer, bytes_buffer,
				keysym_return, status_return);
	if (*status_return == XBufferOverflow)
	    buffer = (char *)realloc(buffer, bytes_buffer = len);
    } while (*status_return == XBufferOverflow);

    return std::string(buffer, len);
}

/****************************************************************************/

Font::Font() : display(NULL), font(0)
{
}

Font::Font(::Display *display, ::Font font) : display(display), font(font)
{
}

Font::operator ::Font() const
{
    return font;
}

int Font::unload()
{
    return XUnloadFont(display, font);
}

/****************************************************************************/

Colormap::Colormap() : display(NULL), colormap(0)
{
}

Colormap::Colormap(::Display *display, ::Colormap colormap)
    : display(display), colormap(colormap)
{
}

Colormap::operator ::Colormap() const
{
    return colormap;
}

Colormap Colormap::copyAndFree()
{
    return Colormap(display, XCopyColormapAndFree(display, colormap));
}

int Colormap::free()
{
    return XFreeColormap(display, colormap);
}

Status Colormap::allocColor(XColor *screen_in_out)
{
    return XAllocColor(display, colormap, screen_in_out);
}

Status Colormap::allocNamedColor(const char *color_name,
				 XColor *screen_def_return,
				 XColor *exact_def_return)
{
    return XAllocNamedColor(display, colormap, color_name, screen_def_return,
			    exact_def_return);
}

Status Colormap::allocColorCells(Bool contig,
				 unsigned long longplane_masks_return[],
				 unsigned int nplanes,
				 unsigned long pixels_return[],
				 unsigned int npixels)
{
    return XAllocColorCells(display, colormap, contig, longplane_masks_return,
			    nplanes, pixels_return, npixels);
}

Status Colormap::allocColorPlanes(Bool contig, unsigned long pixels_return[],
				  int ncolors,
				  int nreds, int ngreens, int nblues,
				  unsigned long *rmask_return,
				  unsigned long *gmask_return,
				  unsigned long *bmask_return)
{
    return XAllocColorPlanes(display, colormap, contig, pixels_return,
			     ncolors, nreds, ngreens, nblues,
			     rmask_return, gmask_return, bmask_return);
}

int Colormap::freeColors(unsigned long pixels[], int npixels,
			 unsigned long planes)
{
    return XFreeColors(display, colormap, pixels, npixels, planes);
}

/****************************************************************************/

Drawable::Drawable() : display(NULL), d(0)
{
}

Drawable::Drawable(::Display *display, ::Drawable d) : display(display), d(d)
{
}

Drawable::operator ::Drawable() const
{
    return d;
}

bool Drawable::isNull() const
{
    return !display;
}

GC Drawable::createGC(unsigned long valuemask, XGCValues *values)
{
    return GC(display, XCreateGC(display, d, valuemask, values));
}

int Drawable::drawArc(::GC gc, int x, int y,
		      unsigned int width, unsigned int height,
		      int angle1, int angle2)
{
    return XDrawArc(display, d, gc, x, y, width, height, angle1, angle2);
}

int Drawable::drawArcs(::GC gc, XArc *arcs, int narcs)
{
    return XDrawArcs(display, d, gc, arcs, narcs);
}

int Drawable::drawImageString(::GC gc, int x, int y, const std::string &string)
{
    return XDrawImageString(display, d, gc, x, y,
			    string.c_str(), string.length());
}

int Drawable::drawImageString16(::GC gc, int x, int y,
				XChar2b *string, int length)
{
    return XDrawImageString16(display, d, gc, x, y, string, length);
}

void Drawable::mbDrawImageString(XFontSet font_set, ::GC gc,
				 int x, int y, const std::string &string)
{
    XmbDrawImageString(display, d, font_set, gc, x, y,
		       string.c_str(), string.length());
}

void Drawable::wcDrawImageString(XFontSet font_set, ::GC gc,
				 int x, int y, const std::wstring &string)
{
    XwcDrawImageString(display, d, font_set, gc, x, y,
		       string.c_str(), string.length());
}

void Drawable::utf8DrawImageString(XFontSet font_set, ::GC gc,
				   int x, int y, const std::string &string)
{
    Xutf8DrawImageString(display, d, font_set, gc, x, y,
			 string.c_str(), string.length());
}

int Drawable::drawLine(::GC gc, int x1, int y1, int x2, int y2)
{
    return XDrawLine(display, d, gc, x1, y1, x2, y2);
}

int Drawable::drawLines(::GC gc, XPoint *points, int npoints, int mode)
{
    return XDrawLines(display, d, gc, points, npoints, mode);
}

int Drawable::drawPoint(::GC gc, int x, int y)
{
    return XDrawPoint(display, d, gc, x, y);
}

int Drawable::drawPoints(::GC gc, XPoint *points, int npoints, int mode)
{
    return XDrawPoints(display, d, gc, points, npoints, mode);
}

int Drawable::drawRectangle(::GC gc, int x, int y,
			    unsigned int width, unsigned int height)
{
    return XDrawRectangle(display, d, gc, x, y, width, height);
}

int Drawable::drawRectangles(::GC gc, XRectangle *rectangles, int nrectangles)
{
    return XDrawRectangles(display, d, gc, rectangles, nrectangles);
}

int Drawable::drawSegments(::GC gc, XSegment *segments, int nsegments)
{
    return XDrawSegments(display, d, gc, segments, nsegments);
}

int Drawable::drawString(::GC gc, int x, int y, const std::string &string)
{
    return XDrawString(display, d, gc, x, y,
		       string.c_str(), string.length());
}

int Drawable::drawString16(::GC gc, int x, int y, XChar2b *string, int length)
{
    return XDrawString16(display, d, gc, x, y, string, length);
}

void Drawable::mbDrawString(XFontSet font_set, ::GC gc,
			    int x, int y, const std::string &string)
{
    XmbDrawString(display, d, font_set, gc, x, y,
		  string.c_str(), string.length());
}

void Drawable::wcDrawString(XFontSet font_set, ::GC gc,
			    int x, int y, const std::wstring &string)
{
    XwcDrawString(display, d, font_set, gc, x, y,
		  string.c_str(), string.length());
}

void Drawable::utf8DrawString(XFontSet font_set, ::GC gc,
			      int x, int y, const std::string &string)
{
    Xutf8DrawString(display, d, font_set, gc, x, y,
		    string.c_str(), string.length());
}

int Drawable::drawText(::GC gc, int x, int y, XTextItem *items, int nitems)
{
    return XDrawText(display, d, gc, x, y, items, nitems);
}

int Drawable::drawText16(::GC gc, int x, int y, XTextItem16 *items, int nitems)
{
    return XDrawText16(display, d, gc, x, y, items, nitems);
}

int Drawable::fillArc(::GC gc, int x, int y,
		      unsigned int width, unsigned int height,
		      int angle1, int angle2)
{
    return XFillArc(display, d, gc, x, y, width, height, angle1, angle2);
}

int Drawable::fillArcs(::GC gc, XArc *arcs, int narcs)
{
    return XFillArcs(display, d, gc, arcs, narcs);
}

int Drawable::fillPolygon(::GC gc, XPoint *points, int npoints,
			  int shape, int mode)
{
    return XFillPolygon(display, d, gc, points, npoints, shape, mode);
}

int Drawable::fillRectangle(::GC gc, int x, int y,
			    unsigned int width, unsigned int height)
{
    return XFillRectangle(display, d, gc, x, y, width, height);
}

int Drawable::fillRectangles(::GC gc, XRectangle *rectangles, int nrectangles)
{
    return XFillRectangles(display, d, gc, rectangles, nrectangles);
}

int Drawable::readBitmapFile(
    const char *filename,
    unsigned int *width_return, unsigned int *height_return,
    Pixmap *bitmap_return, int *x_hot_return, int *y_hot_return)
{
    ::Pixmap pixmap = *bitmap_return;
    int res = XReadBitmapFile(display, d, filename,
			      width_return, height_return,
			      &pixmap, x_hot_return, y_hot_return);
    *bitmap_return = Pixmap(display, pixmap);
    return res;
}

Pixmap Drawable::createPixmapFromBitmapData(
    char *data, unsigned int width, unsigned int height,
    unsigned long fg, unsigned int bg, unsigned int depth)
{
    return Pixmap(display, XCreatePixmapFromBitmapData(
		      display, d, data, width, height, fg, bg, depth));
}

Pixmap Drawable::createBitmapFromData(
    const char *data, unsigned int width, unsigned int height)
{
    return Pixmap(display, XCreateBitmapFromData(
		      display, d, data, width, height));
}

int Drawable::copyArea(
    Drawable src, Drawable dest, ::GC gc, int src_x, int src_y,
    unsigned int width, unsigned int height, int dest_x, int dest_y)
{
    return XCopyArea(src.display, src, dest, gc, src_x, src_y,
		     width, height, dest_x, dest_y);
}

int Drawable::copyPlane(
    Drawable src, Drawable dest, ::GC gc, int src_x, int src_y,
    unsigned int width, unsigned int height, int dest_x, int dest_y,
    unsigned long plane)
{
    return XCopyPlane(src.display, src, dest, gc, src_x, src_y,
		      width, height, dest_x, dest_y, plane);
}

Pixmap Drawable::createPixmap(unsigned int width, unsigned int height,
			      unsigned int depth)
{
    return Pixmap(display, XCreatePixmap(display, d, width, height, depth));
}

Status Drawable::queryBestCursor(unsigned int width, unsigned int height,
				 unsigned int *width_return,
				 unsigned int *height_return)
{
    return XQueryBestCursor(display, d, width, height,
			    width_return, height_return);
}

int Drawable::putImage(::GC gc, XImage *image,
		       int src_x, int src_y, int dest_x, int dest_y,
		       unsigned int width, unsigned int height)
{
    return XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y,
		     width, height);
}

XImage *Drawable::getImage(int x, int y,
			   unsigned int width, unsigned int height,
			   unsigned long plane_mask, int format)
{
    return XGetImage(display, d, x, y, width, height, plane_mask, format);
}

XImage *Drawable::getSubImage(int x, int y,
			      unsigned int width, unsigned int height,
			      unsigned long plane_mask, int format,
			      XImage *dest_image, int dest_x, int dest_y)
{
    return XGetSubImage(display, d, x, y, width, height, plane_mask, format,
			dest_image, dest_x, dest_y);
}

/****************************************************************************/

Window::Window() : display(NULL), w(0)
{
}

Window::Window(::Display *display, ::Window w) : Drawable(display, w),
						 display(display), w(w)
{
}

Window::operator ::Window() const
{
    return w;
}

int Window::changeAttributes(unsigned long valuemask,
			     XSetWindowAttributes *attributes)
{
    return XChangeWindowAttributes(display, w, valuemask, attributes);
}

int Window::setBackground(unsigned long background_pixel)
{
    return XSetWindowBackground(display, w, background_pixel);
}

int Window::setBackgroundPixmap(::Pixmap background_pixmap)
{
    return XSetWindowBackgroundPixmap(display, w, background_pixmap);
}

int Window::setBorder(unsigned long border_pixel)
{
    return XSetWindowBorder(display, w, border_pixel);
}

int Window::setBorderPixmap(::Pixmap border_pixmap)
{
    return XSetWindowBorderPixmap(display, w, border_pixmap);
}

int Window::setColormap(::Colormap colormap)
{
    return XSetWindowColormap(display, w, colormap);
}

int Window::clearArea(int x, int y, unsigned width, unsigned height,
		      bool exposures)
{
    return XClearArea(display, w, x, y, width, height, exposures);
}

int Window::clear()
{
    return XClearWindow(display, w);
}

Window Window::createWindow(
    int x, int y, unsigned int width, unsigned int height,
    unsigned int border_width, int depth,
    unsigned int _class, Visual *visual,
    unsigned long valuemask, XSetWindowAttributes *attributes)
{
    return Window(display,
		  XCreateWindow(display, w, x, y, width, height,
				border_width, depth, _class, visual,
				valuemask, attributes));
}

Window Window::createSimpleWindow(
    int x, int y, unsigned int width, unsigned int height,
    unsigned int border_width, unsigned long border,
    unsigned long background)
{
    return Window(display,
		  XCreateSimpleWindow(display, w, x, y, width, height,
				      border_width, border, background));
}

int Window::selectInput(long event_mask)
{
    return XSelectInput(display, w, event_mask);
}

int Window::map()
{
    return XMapWindow(display, w);
}

int Window::mapRaised()
{
    return XMapRaised(display, w);
}

int Window::mapSubwindows()
{
    return XMapSubwindows(display, w);
}

int Window::unmap()
{
    return XUnmapWindow(display, w);
}

int Window::unmapSubwindows()
{
    return XUnmapSubwindows(display, w);
}

int Window::event(long event_mask, XEvent *event_return)
{
    return XWindowEvent(display, w, event_mask, event_return);
}

bool Window::checkEvent(long event_mask, XEvent *event_return)
{
    return XCheckWindowEvent(display, w, event_mask, event_return);
}

bool Window::checkTypedEvent(int event_type, XEvent *event_return)
{
    return XCheckTypedWindowEvent(display, w, event_type, event_return);
}

Colormap Window::createColormap(Visual *visual, int alloc)
{
    return Colormap(display, XCreateColormap(display, w, visual, alloc));
}

int Window::defineCursor(::Cursor cursor)
{
    return XDefineCursor(display, w, cursor);
}

int Window::undefineCursor()
{
    return XUndefineCursor(display, w);
}

int Window::configure(unsigned value_mask, XWindowChanges *changes)
{
    return XConfigureWindow(display, w, value_mask, changes);
}

int Window::move(int x, int y)
{
    return XMoveWindow(display, w, x, y);
}

int Window::resize(unsigned width, unsigned height)
{
    return XResizeWindow(display, w, width, height);
}

int Window::moveResize(int x, int y, unsigned width, unsigned height)
{
    return XMoveResizeWindow(display, w, x, y, width, height);
}

int Window::setBorderWidth(unsigned width)
{
    return XSetWindowBorderWidth(display, w, width);
}

int Window::destroy()
{
    return XDestroyWindow(display, w);
}

int Window::destroySubwindows()
{
    return XDestroySubwindows(display, w);
}

int Window::grabPointer(bool owner_events, unsigned int event_mask,
			int pointer_mode, int keyboard_mode,
			::Window confine_to, ::Cursor cursor, Time time)
{
    return XGrabPointer(display, w, owner_events, event_mask, pointer_mode,
			keyboard_mode, confine_to, cursor, time);
}

int Window::getProperty(
    Atom property, long long_offset, long long_length, Bool _delete,
    Atom req_type, Atom *actual_type_return, int *actual_format_return,
    unsigned long *nitems_return, unsigned long *bytes_after_return,
    unsigned char **prop_return)
{
    return XGetWindowProperty(
	display, w, property, long_offset, long_length, _delete, req_type,
	actual_type_return, actual_format_return, nitems_return,
	bytes_after_return, prop_return);
}

Atom *Window::listProperties(int *num_prop_return)
{
    return XListProperties(display, w, num_prop_return);
}

int Window::changeProperty(Atom property, Atom type, int format, int mode,
			   const unsigned char *data, int nelements)
{
    return XChangeProperty(display, w, property, type, format, mode,
			   data, nelements);
}

int Window::rotateProperties(Atom properties[], int num_prop, int npositions)
{
    return XRotateWindowProperties(display, w, properties, num_prop,
				   npositions);
}

int Window::deleteProperty(Atom property)
{
    return XDeleteProperty(display, w, property);
}

void Window::setTextProperty(XTextProperty *text_prop, Atom property)
{
    XSetTextProperty(display, w, text_prop, property);
}

Status Window::getTextProperty(XTextProperty *text_prop_return, Atom property)
{
    return XGetTextProperty(display, w, text_prop_return, property);
}

void Window::setWMProperties(XTextProperty *window_name,
			     XTextProperty *icon_name,
			     char **argv, int argc,
			     XSizeHints *normal_hints, XWMHints *wm_hints,
			     XClassHint *class_hints)
{
    XSetWMProperties(display, w, window_name, icon_name, argv, argc,
		     normal_hints, wm_hints, class_hints);
}

void Window::mbSetWMProperties(const std::string &window_name,
			       const std::string &icon_name,
			       char *argv[], int argc,
			       XSizeHints *normal_hints, XWMHints *wm_hints,
			       XClassHint *class_hints)
{
    XmbSetWMProperties(display, w, window_name.c_str(), icon_name.c_str(),
		       argv, argc, normal_hints, wm_hints, class_hints);
}

void Window::utf8SetWMProperties(const std::string &window_name,
				 const std::string &icon_name,
				 char *argv[], int argc,
				 XSizeHints *normal_hints, XWMHints *wm_hints,
				 XClassHint *class_hints)
{
    Xutf8SetWMProperties(display, w, window_name.c_str(), icon_name.c_str(),
			 argv, argc, normal_hints, wm_hints, class_hints);
}

void Window::setWMName(XTextProperty *text_prop)
{
    XSetWMName(display, w, text_prop);
}

Status Window::getWMName(XTextProperty *text_prop_return)
{
    return XGetWMName(display, w, text_prop_return);
}

int Window::storeName(const char *window_name)
{
    return XStoreName(display, w, window_name);
}

Status Window::fetchName(char **window_name_return)
{
    return XFetchName(display, w, window_name_return);
}

void Window::setWMIconName(XTextProperty *text_prop)
{
    XSetWMIconName(display, w, text_prop);
}

Status Window::getWMIconName(XTextProperty *text_prop_return)
{
    return XGetWMIconName(display, w, text_prop_return);
}

int Window::setIconName(const char *icon_name)
{
    return XSetIconName(display, w, icon_name);
}

Status Window::getIconName(char **icon_name_return)
{
    return XGetIconName(display, w, icon_name_return);
}

void Window::setWMNormalHints(XSizeHints *hints)
{
    XSetWMNormalHints(display, w, hints);
}

Status Window::getWMNormalHints(XSizeHints *hints_return,
				long *supplied_return)

{
    return XGetWMNormalHints(display, w, hints_return, supplied_return);
}

void Window::setWMSizeHints(XSizeHints *hints, Atom property)
{
    XSetWMSizeHints(display, w, hints, property);
}

Status Window::getWMSizeHints(XSizeHints *hints_return, long *supplied_return,
			      Atom property)
{
    return XGetWMSizeHints(display, w, hints_return,
			   supplied_return, property);
}

int Window::setWMHints(XWMHints *wmhints)
{
    return XSetWMHints(display, w, wmhints);
}

XWMHints *Window::getWMHints()
{
    return XGetWMHints(display, w);
}

int Window::setClassHint(XClassHint *class_hints)
{
    return XSetClassHint(display, w, class_hints);
}

Status Window::getClassHint(XClassHint *class_hints_return)
{
    return XGetClassHint(display, w, class_hints_return);
}

int Window::setTransientForHint(::Window prop_window)
{
    return XSetTransientForHint(display, w, prop_window);
}

Status Window::getTransientForHint(::Window *prop_window_return)
{
    return XGetTransientForHint(display, w, prop_window_return);
}

Status Window::setWMProtocols(Atom *protocols, int count)
{
    return XSetWMProtocols(display, w, protocols, count);
}

Status Window::getWMProtocols(Atom **protocols_return, int *count_return)
{
    return XGetWMProtocols(display, w, protocols_return, count_return);
}

Status Window::sendEvent(Bool propagate, long event_mask, XEvent *event_send)
{
    return XSendEvent(display, w, propagate, event_mask, event_send);
}

/****************************************************************************/

Pixmap::Pixmap() : display(NULL), pixmap(0)
{
}

Pixmap::Pixmap(::Display *display, ::Pixmap pixmap)
    : Drawable(display, pixmap), display(display), pixmap(pixmap)
{
}

Pixmap::operator ::Pixmap() const
{
    return pixmap;
}

int Pixmap::writeBitmapFile(const char *filename,
			    unsigned int width, unsigned int height,
			    int x_hot, int y_hot)
{
    return XWriteBitmapFile(display, filename, pixmap,
			    width, height, x_hot, y_hot);
}

int Pixmap::free()
{
    return XFreePixmap(display, pixmap);
}

/****************************************************************************/

Cursor::Cursor() : display(NULL), cursor(0)
{
}

Cursor::Cursor(::Display *display, ::Cursor cursor)
    : Drawable(display, cursor), display(display), cursor(cursor)
{
}

Cursor::operator ::Cursor() const
{
    return cursor;
}

int Cursor::recolor(XColor *foreground_color, XColor *background_color)
{
    return XRecolorCursor(display, cursor, foreground_color, background_color);
}

int Cursor::free()
{
    return XFreeCursor(display, cursor);
}

} /* namespace X */
